@@ -7,13 +7,13 @@
  *
  * Based on the FPDF class by Olivier Plathey (http://www.fpdf.org).
  *
- * Copyright 2003-2005 Olivier Plathey <olivier@fpdf.org>
- * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
+ * Copyright 2003-2007 Olivier Plathey <olivier@fpdf.org>
+ * Copyright 2003-2007 Marko Djukic <marko@oblo.com>
  *
  * See the enclosed file COPYING for license information (LGPL). If you did not
  * receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * $Horde: framework/File_PDF/PDF.php,v 1.18.10.10 2005/07/07 18:16:36 selsky Exp $
+ * $Horde: framework/File_PDF/PDF.php,v 1.18.10.17 2007/01/02 13:54:16 jan Exp $
  *
  * @author  Olivier Plathey <olivier@fpdf.org>
  * @author  Marko Djukic <marko@oblo.com>
@@ -455,7 +455,8 @@ class File_PDF {
     {
         /* Check for PHP locale-related bug. */
         if (1.1 == 1) {
-            return File_PDF::raiseError('Do not alter the locale before including the class file.');
+            $error = File_PDF::raiseError('Do not alter the locale before including the class file.');
+            return $error;
         }
 
         /* Default parameters. */
@@ -487,7 +488,8 @@ class File_PDF {
         } elseif ($params['unit'] == 'in') {
             $pdf->_scale = 72;
         } else {
-            return File_PDF::raiseError(sprintf('Incorrect units: %s', $params['unit']));
+            $error = File_PDF::raiseError(sprintf('Incorrect units: %s', $params['unit']));
+            return $error;
         }
         /* Page format. */
         if (is_string($params['format'])) {
@@ -503,7 +505,8 @@ class File_PDF {
             } elseif ($params['format'] == 'legal') {
                 $params['format'] = array(612, 1008);
             } else {
-                return File_PDF::raiseError(sprintf('Unknown page format: %s', $params['format']));
+                $error = File_PDF::raiseError(sprintf('Unknown page format: %s', $params['format']));
+                return $error;
             }
             $pdf->fwPt = $params['format'][0];
             $pdf->fhPt = $params['format'][1];
@@ -525,7 +528,8 @@ class File_PDF {
             $pdf->wPt = $pdf->fhPt;
             $pdf->hPt = $pdf->fwPt;
         } else {
-            return File_PDF::raiseError(sprintf('Incorrect orientation: %s', $params['orientation']));
+            $error = File_PDF::raiseError(sprintf('Incorrect orientation: %s', $params['orientation']));
+            return $error;
         }
         $pdf->_current_orientation = $pdf->_default_orientation;
         $pdf->w = $pdf->wPt / $pdf->_scale;
@@ -635,6 +639,30 @@ class File_PDF {
         $this->_right_margin = $margin;
     }
 
+    /**
+     * Returns the actual page width.
+     *
+     * @since File_PDF 0.2.0
+     *
+     * @return float  The page width.
+     */
+    function getPageWidth()
+    {
+    	return ($this->w - $this->_right_margin - $this->_left_margin);
+    }
+
+    /**
+     * Returns the actual page height.
+     *
+     * @since File_PDF 0.2.0
+     *
+     * @return float  The page height.
+     */
+    function getPageHeight()
+    {
+    	return ($this->h - $this->_top_margin - $this->_break_margin);
+    }
+
     /**
      * Enables or disables the automatic page breaking mode. When enabling,
      * the second parameter is the distance from the bottom of the page that
@@ -828,8 +856,10 @@ class File_PDF {
      */
     function addPage($orientation = '')
     {
-        $style = $this->_font_style . ($this->_underline ? 'U' : '');
-        $size  = $this->_font_size_pt;
+        /* For good measure make sure this is called. */
+        $this->_beginDoc();
+
+        /* Save style settings so that they are not overridden by footer(). */
         $lw = $this->_line_width;
         $dc = $this->_draw_color;
         $fc = $this->_fill_color;
@@ -848,9 +878,13 @@ class File_PDF {
         /* Set line width. */
         $this->_line_width = $lw;
         $this->_out(sprintf('%.2f w', $lw * $this->_scale));
-        /* Set font. */
+        /* Set font for the beginning of the page. */
+        $font_family = null;
         if ($this->_font_family) {
-            $this->setFont($this->_font_family, $style, $this->_font_size_pt);
+            $font_family = $this->_font_family;
+            $font_style = $this->_font_style . ($this->_underline ? 'U' : '');
+            $font_size  = $this->_font_size_pt;
+            $this->setFont($font_family, $font_style, $font_size);
         }
         /* Set colors. */
         $this->_fill_color = $fc;
@@ -870,9 +904,10 @@ class File_PDF {
             $this->_line_width = $lw;
             $this->_out(sprintf('%.2f w', $lw * $this->_scale));
         }
-        /* Restore font. */
-        if ($this->_font_family) {
-            $this->setFont($this->_font_family, $style, $this->_font_size_pt);
+        /* Make sure the font is set for this page as it was before the
+         * header. */
+        if ($font_family) {
+            $this->setFont($font_family, $font_style, $font_size, true);
         }
         /* Restore colors. */
         if ($this->_draw_color != $dc) {
@@ -1309,6 +1344,10 @@ class File_PDF {
      *                        current size. If no size has been specified since
      *                        the beginning of the document, the value taken
      *                        is 12.
+     * @param boolean $force  Force the setting of the font. Each new page will
+     *                        require a new call to File_PDF::setFont and
+     *                        settings this to true will make sure that the
+     *                        checks for same font calls will be skipped.
      *
      * @see File_PDF::addFont
      * @see File_PDF::setFontSize
@@ -1316,7 +1355,7 @@ class File_PDF {
      * @see File_PDF::multiCell
      * @see File_PDF::Write
      */
-    function setFont($family, $style = '', $size = null)
+    function setFont($family, $style = '', $size = null, $force = false)
     {
         $family = strtolower($family);
         if ($family == 'arial') {
@@ -1347,10 +1386,11 @@ class File_PDF {
             $size = $this->_font_size_pt;
         }
 
-        /* If font requested is already the current font don't bother with the
-         * rest of the function, simply return. */
-        if ($this->_font_family == $family && $this->_font_style == $style
-                && $this->_font_size_pt == $size) {
+        /* If font requested is already the current font and no force setting
+         * of the font is requested (eg. when adding a new page) don't bother
+         * with the rest of the function and simply return. */
+        if ($this->_font_family == $family && $this->_font_style == $style &&
+            $this->_font_size_pt == $size && !$force) {
             return;
         }
 
@@ -2009,6 +2049,9 @@ class File_PDF {
             } else {
                 return $this->raiseError(sprintf('Unsupported image file type: %s', $type));
             }
+            if (is_a($info, 'PEAR_Error')) {
+                return $info;
+            }
             set_magic_quotes_runtime($mqr);
             $info['i'] = count($this->_images) + 1;
             $this->_images[$file] = $info;
@@ -2245,9 +2288,11 @@ class File_PDF {
 
     function _beginDoc()
     {
-        /* Start document. */
-        $this->_state = 1;
-        $this->_out('%PDF-1.3');
+        /* Start document, but only if not yet started. */
+        if ($this->_state < 1) {
+            $this->_state = 1;
+            $this->_out('%PDF-1.3');
+        }
     }
 
     function _putPages()
@@ -2569,7 +2614,6 @@ class File_PDF {
         $this->x = $this->_left_margin;
         $this->y = $this->_top_margin;
         $this->_last_height = 0;
-        $this->_font_family = '';
         /* Page orientation */
         if (!$orientation) {
             $orientation = $this->_default_orientation;
