@@ -7,19 +7,27 @@
  *   'phptype'      The database type (ie. 'pgsql', 'mysql', etc.).</pre>
  *
  * Optional parameters:<pre>
- *   'encryption'       The encryption to use to store the password in the
- *                      table (e.g. plain, crypt, md5-hex, md5-base64, smd5,
- *                      sha, ssha, aprmd5).
- *                      DEFAULT: 'md5-hex'
- *   'show_encryption'  Whether or not to prepend the encryption in the
- *                      password field.
- *                      DEFAULT: 'false'
- *   'password_field'   The name of the password field in the auth table.
- *                      DEFAULT: 'user_pass'
- *   'table'            The name of the SQL table to use in 'database'.
- *                      DEFAULT: 'horde_users'
- *   'username_field'   The name of the username field in the auth table.
- *                      DEFAULT: 'user_uid'</pre>
+ *   'encryption'             The encryption to use to store the password in
+ *                            the table (e.g. plain, crypt, md5-hex,
+ *                            md5-base64, smd5, sha, ssha, aprmd5).
+ *                            DEFAULT: 'md5-hex'
+ *   'show_encryption'        Whether or not to prepend the encryption in the
+ *                            password field.
+ *                            DEFAULT: 'false'
+ *   'password_field'         The name of the password field in the auth table.
+ *                            DEFAULT: 'user_pass'
+ *   'table'                  The name of the SQL table to use in 'database'.
+ *                            DEFAULT: 'horde_users'
+ *   'username_field'         The name of the username field in the auth table.
+ *                            DEFAULT: 'user_uid'
+ *   'soft_expiration_field'  The name of the field containing a date after
+ *                            which the system will request the user change his
+ *                            or her password.
+ *                            DEFAULT: none
+ *   'hard_expiration_field'  The name of the field containing a date after
+ *                            which the account is no longer valid and the user
+ *                            will not be able to log in at all.
+ *                            DEFAULT: none</pre>
  *
  * Required by some database implementations:<pre>
  *   'hostspec'     The hostname of the database server.
@@ -34,9 +42,9 @@
  * The table structure for the Auth system is in
  * scripts/sql/horde_users.sql.
  *
- * $Horde: framework/Auth/Auth/sql.php,v 1.69.10.13 2005/07/03 04:26:58 selsky Exp $
+ * $Horde: framework/Auth/Auth/sql.php,v 1.69.10.20 2007/01/02 13:54:07 jan Exp $
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -55,7 +63,7 @@ class Auth_sql extends Auth {
      */
     var $capabilities = array('add'           => true,
                               'update'        => true,
-                              'resetpassword' => false,
+                              'resetpassword' => true,
                               'remove'        => true,
                               'list'          => true,
                               'transparent'   => false);
@@ -100,8 +108,7 @@ class Auth_sql extends Auth {
         $this->_connect();
 
         /* Build the SQL query. */
-        $query = sprintf('SELECT %s FROM %s WHERE %s = ?',
-                         $this->_params['password_field'],
+        $query = sprintf('SELECT * FROM %s WHERE %s = ?',
                          $this->_params['table'],
                          $this->_params['username_field']);
         $values = array($userId);
@@ -109,22 +116,40 @@ class Auth_sql extends Auth {
         Horde::logMessage('SQL Query by Auth_sql::_authenticate(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
         $result = $this->_db->query($query, $values);
-        if (!is_a($result, 'PEAR_Error')) {
-            $row = $result->fetchRow(DB_GETMODE_ASSOC);
-            if (is_array($row) && $this->_comparePasswords($row[$this->_params['password_field']], $credentials['password'])) {
-                $result->free();
-                return true;
-            } else {
-                if (is_array($row)) {
-                    $result->free();
-                }
-                $this->_setAuthError(AUTH_REASON_BADLOGIN);
-                return false;
-            }
-        } else {
+        if (is_a($result, 'PEAR_Error')) {
             $this->_setAuthError(AUTH_REASON_FAILED);
             return false;
         }
+
+        $row = $result->fetchRow(DB_GETMODE_ASSOC);
+        if (is_array($row)) {
+            $result->free();
+        } else {
+            $this->_setAuthError(AUTH_REASON_BADLOGIN);
+            return false;
+        }
+
+        if (!$this->_comparePasswords($row[$this->_params['password_field']],
+                                      $credentials['password'])) {
+            $this->_setAuthError(AUTH_REASON_BADLOGIN);
+            return false;
+        }
+
+        $now = time();
+        if (!empty($this->_params['hard_expiration_field']) &&
+            !empty($row[$this->_params['hard_expiration_field']]) &&
+            ($now > $row[$this->_params['hard_expiration_field']])) {
+            $this->_setAuthError(AUTH_REASON_EXPIRED);
+            return false;
+        }
+
+        if (!empty($this->_params['soft_expiration_field']) &&
+            !empty($row[$this->_params['soft_expiration_field']]) &&
+            ($now > $row[$this->_params['soft_expiration_field']])) {
+            $this->_authCredentials['changeRequested'] = true;
+        }
+
+        return true;
     }
 
     /**
@@ -175,20 +200,58 @@ class Auth_sql extends Auth {
         $this->_connect();
 
         /* Build the SQL query. */
-        $query = sprintf('UPDATE %s SET %s = ?, %s = ? WHERE %s = ?',
-                         $this->_params['table'],
-                         $this->_params['username_field'],
-                         $this->_params['password_field'],
-                         $this->_params['username_field']);
-        $values = array($newID, $this->getCryptedPassword($credentials['password'],
-                                                          '',
-                                                          $this->_params['encryption'],
-                                                          $this->_params['show_encryption']), $oldID);
+        $tuple = array();
+        $tuple[$this->_params['username_field']] = $newID;
+        $tuple[$this->_params['password_field']] =
+            $this->getCryptedPassword($credentials['password'],
+                                      '',
+                                      $this->_params['encryption'],
+                                      $this->_params['show_encryption']);
 
-        Horde::logMessage('SQL Query by Auth_sql::updateUser(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        if (empty($this->_params['soft_expiration_window'])) {
+            if (!empty($this->_params['soft_expiration_field'])) {
+                $tuple[$this->_params['soft_expiration_field']] = null;
+            }
+        } else {
+            $date = time();
+            $datea = localtime($date, true);
+            $date = mktime($datea['tm_hour'], $datea['tm_min'],
+                           $datea['tm_sec'], $datea['tm_mon'] + 1,
+                           $datea['tm_mday'] + $this->_params['soft_expiration_window'],
+                           $datea['tm_year']);
+
+            $tuple[$this->_params['soft_expiration_field']] = $date;
+
+            global $notification;
+            if (!empty($notification)) {
+                $notification->push(strftime(_("New password will expire on %x."), $date), 'horde.message');
+            }
+
+            if (empty($this->_params['hard_expiration_window'])) {
+                $tuple[$this->_params['hard_expiration_field']] = null;
+            } else {
+                $datea = localtime($date, true);
+                $date = mktime($datea['tm_hour'], $datea['tm_min'],
+                               $datea['tm_sec'], $datea['tm_mon'] + 1,
+                               $datea['tm_mday'] + $this->_params['soft_expiration_window'],
+                               $datea['tm_year']);
+
+                $tuple[$this->_params['hard_expiration_field']] = $date;
+            }
+        }
+
+        require_once 'Horde/SQL.php';
+        $query = sprintf('UPDATE %s SET %s WHERE %s = ?',
+                         $this->_params['table'],
+                         Horde_SQL::updateValues($this->_db, $tuple),
+                         $this->_params['username_field']);
+        $values = array($oldID);
+
+        Horde::logMessage('SQL Query by Auth_sql:updateUser(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
         $result = $this->_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
         }
 
@@ -201,7 +264,7 @@ class Auth_sql extends Auth {
      *
      * @param string $user_id  The user id for which to reset the password.
      *
-     * @return mixed  The new passwrd on success or a PEAR_Error object on
+     * @return mixed  The new password on success or a PEAR_Error object on
      *                failure.
      */
     function resetPassword($user_id)
@@ -220,7 +283,8 @@ class Auth_sql extends Auth {
         $values = array($this->getCryptedPassword($password,
                                                   '',
                                                   $this->_params['encryption'],
-                                                  $this->_params['show_encryption']), $user_id);
+                                                  $this->_params['show_encryption']),
+                        $user_id);
 
         Horde::logMessage('SQL Query by Auth_sql::resetPassword(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
@@ -306,6 +370,8 @@ class Auth_sql extends Auth {
      * Compare an encrypted password to a plaintext string to see if
      * they match.
      *
+     * @access private
+     *
      * @param string $encrypted  The crypted password to compare against.
      * @param string $plaintext  The plaintext password to verify.
      *
@@ -358,9 +424,13 @@ class Auth_sql extends Auth {
         }
         if (empty($this->_params['username_field'])) {
             $this->_params['username_field'] = 'user_uid';
+        } else {
+            $this->_params['username_field'] = String::lower($this->_params['username_field']);
         }
         if (empty($this->_params['password_field'])) {
             $this->_params['password_field'] = 'user_pass';
+        } else {
+            $this->_params['password_field'] = String::lower($this->_params['password_field']);
         }
 
         /* Connect to the SQL server using the supplied parameters. */
@@ -371,7 +441,14 @@ class Auth_sql extends Auth {
             Horde::fatal(_("Unable to connect to SQL server."), __FILE__, __LINE__);
         }
 
-        $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+        // Set DB portability options.
+        switch ($this->_db->phptype) {
+        case 'mssql':
+            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
+            break;
+        default:
+            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+        }
 
         $this->_connected = true;
         return true;
