@@ -1,5 +1,6 @@
 <?php
 
+/** Horde_String */
 require_once 'Horde/String.php';
 
 /**
@@ -9,10 +10,10 @@ require_once 'Horde/String.php';
  * Browser identification is performed by examining the HTTP_USER_AGENT
  * environment variable provided by the web server.
  *
- * $Horde: framework/Browser/Browser.php,v 1.153.2.25.2.1 2005/10/26 06:49:29 slusarz Exp $
+ * $Horde: framework/Browser/Browser.php,v 1.153.2.52 2007/02/02 04:00:22 chuck Exp $
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2007 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -85,6 +86,7 @@ class Browser {
         'Slurp',
         'Yahoo',
         /* The rest alphabetically. */
+        'appie',
         'Arachnoidea',
         'ArchitextSpider',
         'Ask Jeeves',
@@ -97,6 +99,8 @@ class Browser {
         'FAST-WebCrawler',
         'FDSE robot',
         'fido',
+        'findlinks',
+        'Francis',
         'geckobot',
         'Gigabot',
         'Girafabot',
@@ -104,12 +108,14 @@ class Browser {
         'Gulliver',
         'HTTrack',
         'ia_archiver',
+        'iCCrawler',
         'InfoSeek',
         'kinjabot',
         'KIT-Fireball',
         'larbin',
         'LEIA',
         'lmspider',
+        'lwp-trivial',
         'Lycos_Spider',
         'Mediapartners-Google',
         'MuscatFerret',
@@ -117,14 +123,21 @@ class Browser {
         'OmniExplorer_Bot',
         'polybot',
         'Pompos',
+        'RufusBot',
         'Scooter',
+        'Seekbot',
+        'sogou spider',
+        'sproose',
         'Teoma',
         'TheSuBot',
         'TurnitinBot',
         'Ultraseek',
         'ViolaBot',
+        'voyager',
+        'W3C-checklink',
         'webbandit',
         'www.almaden.ibm.com/cs/crawler',
+        'yacy',
         'ZyBorg',
     );
 
@@ -270,12 +283,26 @@ class Browser {
         if (!empty($this->_agent)) {
             $this->_setPlatform();
 
-            if (preg_match('|Opera[/ ]([0-9.]+)|', $this->_agent, $version)) {
+            if (strpos($this->_lowerAgent, 'mobileexplorer') !== false ||
+                strpos($this->_lowerAgent, 'openwave') !== false ||
+                strpos($this->_lowerAgent, 'opera mini') !== false ||
+                strpos($this->_lowerAgent, 'operamini') !== false) {
+                $this->setFeature('frames', false);
+                $this->setFeature('javascript', false);
+                $this->setQuirk('avoid_popup_windows');
+                $this->_mobile = true;
+            } elseif (preg_match('|Opera[/ ]([0-9.]+)|', $this->_agent, $version)) {
                 $this->setBrowser('opera');
                 list($this->_majorVersion, $this->_minorVersion) = explode('.', $version[1]);
                 $this->setFeature('javascript', true);
                 $this->setQuirk('no_filename_spaces');
 
+                /* Opera Mobile reports its screen resolution in the
+                 * user agent strings. */
+                if (preg_match('/; (120x160|240x280|240x320|320x320)\)/', $this->_agent)) {
+                    $this->_mobile = true;
+                }
+
                 if ($this->_majorVersion >= 7) {
                     $this->setFeature('dom');
                     $this->setFeature('iframes');
@@ -307,23 +334,24 @@ class Browser {
                     $this->_minorVersion = 0;
                 }
 
-                /* IE on Windows does not support alpha transparency in PNG
-                 * images. */
-                if (preg_match('/windows/i', $this->_agent)) {
+                /* IE (< 7) on Windows does not support alpha transparency in
+                 * PNG images. */
+                if (($this->_majorVersion < 7) &&
+                    preg_match('/windows/i', $this->_agent)) {
                     $this->setQuirk('png_transparency');
                 }
 
-                /* IE 6 (pre-SP1) and 5.5 (pre-SP1) has buggy compression.
+                /* IE 6 (pre-SP1) and 5.5 (pre-SP1) have buggy compression.
                  * The versions affected are as follows:
                  * 6.00.2462.0000  Internet Explorer 6 Public Preview (Beta)
                  * 6.00.2479.0006  Internet Explorer 6 Public Preview (Beta)
                                     Refresh
                  * 6.00.2600.0000  Internet Explorer 6 (Windows XP)
-                 * 5.50.3825.1300   Internet Explorer 5.5 Developer Preview (Beta)
-                 * 5.50.4030.2400   Internet Explorer 5.5 & Internet Tools Beta
-                 * 5.50.4134.0100   Internet Explorer 5.5 for Windows Me (4.90.3000)
-                 * 5.50.4134.0600   Internet Explorer 5.5
-                 * 5.50.4308.2900   Internet Explorer 5.5 Advanced Security Privacy Beta
+                 * 5.50.3825.1300  Internet Explorer 5.5 Developer Preview (Beta)
+                 * 5.50.4030.2400  Internet Explorer 5.5 & Internet Tools Beta
+                 * 5.50.4134.0100  Internet Explorer 5.5 for Windows Me (4.90.3000)
+                 * 5.50.4134.0600  Internet Explorer 5.5
+                 * 5.50.4308.2900  Internet Explorer 5.5 Advanced Security Privacy Beta
                  *
                  * See:
                  * ====
@@ -349,6 +377,18 @@ class Browser {
                 }
 
                 switch ($this->_majorVersion) {
+                case 7:
+                    $this->setFeature('javascript', 1.4);
+                    $this->setFeature('dom');
+                    $this->setFeature('iframes');
+                    $this->setFeature('utf');
+                    $this->setFeature('rte');
+                    $this->setFeature('homepage');
+                    $this->setFeature('accesskey');
+                    $this->setFeature('optgroup');
+                    $this->setFeature('xmlhttpreq');
+                    break;
+
                 case 6:
                     $this->setFeature('javascript', 1.4);
                     $this->setFeature('dom');
@@ -430,7 +470,6 @@ class Browser {
                     if ($this->_majorVersion > 125 ||
                         ($this->_majorVersion == 125 &&
                          $this->_minorVersion >= 1)) {
-                        $this->setFeature('utf');
                         $this->setFeature('accesskey');
                         $this->setFeature('xmlhttpreq');
                     }
@@ -547,7 +586,14 @@ class Browser {
                 $this->setBrowser('imode');
                 $this->setFeature('images', false);
                 $this->_mobile = true;
-            } elseif (strpos($this->_agent, 'MOT-V3/') !== false) {
+        } elseif (strpos($this->_agent, 'BlackBerry') !== false) {
+                $this->setBrowser('blackberry');
+                $this->setFeature('html', false);
+                $this->setFeature('javascript', false);
+                $this->setFeature('hdml');
+                $this->setFeature('wml');
+                $this->_mobile = true;
+            } elseif (strpos($this->_agent, 'MOT-') !== false) {
                 $this->setBrowser('motorola');
                 $this->setFeature('html', false);
                 $this->setFeature('javascript', false);
@@ -951,16 +997,15 @@ class Browser {
             }
         }
 
-        /* Content-Length Header. Don't send Content-Length for
-         * HTTP/1.1 servers. */
-        if (($this->getHTTPProtocol() != '1.1') && !is_null($cLength)) {
+        /* Content-Length Header. */
+        if (!is_null($cLength)) {
             header('Content-Length: ' . $cLength);
         }
 
         /* Overwrite Pragma: and other caching headers for IE. */
         if ($this->hasQuirk('cache_ssl_downloads')) {
             header('Expires: 0');
-            header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
+            header('Cache-Control: must-revalidate');
             header('Pragma: public');
         }
     }
@@ -1010,13 +1055,13 @@ class Browser {
             return false;
         }
 
-        return (in_array($subtype, $this->_images));
+        return in_array($subtype, $this->_images);
     }
 
     /**
      * Escape characters in javascript code if the browser requires it.
-     * %23, %26, and %2B (for IE) and %27 need to be escaped or else
-     * jscript will interpret it as a single quote, pound sign, or
+     * %23, %26, and %2B (for some browsers) and %27 need to be escaped or
+     * else javascript will interpret it as a single quote, pound sign, or
      * ampersand and refuse to work.
      *
      * @param string $code  The JS code to escape.
@@ -1028,9 +1073,10 @@ class Browser {
         $from = $to = array();
 
         if ($this->isBrowser('msie') ||
-            ($this->isBrowser('mozilla') && ($this->getMajor() >= 5))) {
+            ($this->isBrowser('mozilla') && ($this->getMajor() >= 5)) ||
+            $this->isBrowser('konqueror')) {
             $from = array('%23', '%26', '%2B');
-            $to = array(urlencode('%23'), urlencode('%26'), urlencode('%2B'));
+            $to = array('%2523', '%2526', '%252B');
         }
         $from[] = '%27';
         $to[] = '\%27';
