@@ -6,9 +6,9 @@ require_once 'Horde/Crypt.php';
  * Horde_Crypt_smime:: provides a framework for Horde applications to
  * interact with the OpenSSL library and implement S/MIME.
  *
- * $Horde: framework/Crypt/Crypt/smime.php,v 1.49.2.8 2005/07/19 04:24:25 slusarz Exp $
+ * $Horde: framework/Crypt/Crypt/smime.php,v 1.49.2.13 2007/01/02 13:54:12 jan Exp $
  *
- * Copyright 2002-2005 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2002-2007 Mike Cochrane <mike@graftonhall.co.nz>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -28,8 +28,8 @@ class Horde_Crypt_smime extends Horde_Crypt {
         '2.5.4.3' => 'CommonName',
         '2.5.4.4' => 'Surname',
         '2.5.4.6' => 'Country',
-        '2.5.4.7' => 'StateOrProvince',
-        '2.5.4.8' => 'Location',
+        '2.5.4.7' => 'Location',
+        '2.5.4.8' => 'StateOrProvince',
         '2.5.4.9' => 'StreetAddress',
         '2.5.4.10' => 'Organisation',
         '2.5.4.11' => 'OrganisationalUnit',
@@ -386,7 +386,7 @@ class Horde_Crypt_smime extends Horde_Crypt {
         if (openssl_pkcs7_encrypt($input, $output, $params['pubkey'], array('To' => $email))) {
             $result = file_get_contents($output);
             if (!empty($result)) {
-                return $result;
+                return $this->_fixContentType($result, 'encrypt');
             }
         }
 
@@ -467,7 +467,8 @@ class Horde_Crypt_smime extends Horde_Crypt {
             return PEAR::raiseError(_("Could not S/MIME sign message."), 'horde.error');
         }
 
-        return file_get_contents($output);
+        $data = file_get_contents($output);
+        return $this->_fixContentType($data, 'signature');
     }
 
     /**
@@ -608,7 +609,7 @@ class Horde_Crypt_smime extends Horde_Crypt {
 
         /* Subject (a/k/a Certificate Owner) */
         if (isset($certificate['subject'])) {
-            $text .= "<b>" . _("Certificate Owner") . ":</b>\n";
+            $text .= "<strong>" . _("Certificate Owner") . ":</strong>\n";
 
             foreach ($certificate['subject'] as $key => $value) {
                 if (isset($fieldnames[$key])) {
@@ -622,7 +623,7 @@ class Horde_Crypt_smime extends Horde_Crypt {
 
         /* Issuer */
         if (isset($certificate['issuer'])) {
-            $text .= "<b>" . _("Issuer") . ":</b>\n";
+            $text .= "<strong>" . _("Issuer") . ":</strong>\n";
 
             foreach ($certificate['issuer'] as $key => $value) {
                 if (isset($fieldnames[$key])) {
@@ -635,13 +636,13 @@ class Horde_Crypt_smime extends Horde_Crypt {
         }
 
         /* Dates  */
-        $text .= "<b>" . _("Validity") . ":</b>\n";
+        $text .= "<strong>" . _("Validity") . ":</strong>\n";
         $text .= sprintf("&nbsp;&nbsp;%s: %s\n", _("Not Before"), strftime("%x %X", $certificate['validity']['notbefore']));
         $text .= sprintf("&nbsp;&nbsp;%s: %s\n", _("Not After"), strftime("%x %X", $certificate['validity']['notafter']));
         $text .= "\n";
 
         /* Certificate Owner - Public Key Info */
-        $text .= "<b>" . _("Public Key Info") . ":</b>\n";
+        $text .= "<strong>" . _("Public Key Info") . ":</strong>\n";
         $text .= sprintf("&nbsp;&nbsp;%s: %s\n", _("Public Key Algorithm"), $certificate['subjectPublicKeyInfo']['algorithm']);
         if ($certificate['subjectPublicKeyInfo']['algorithm'] == 'rsaEncryption') {
             if (Util::extensionExists('bcmath')) {
@@ -679,7 +680,7 @@ class Horde_Crypt_smime extends Horde_Crypt {
 
         /* X509v3 extensions */
         if (isset($certificate['extensions'])) {
-            $text .= "<b>" . _("X509v3 extensions") . ":</b>\n";
+            $text .= "<strong>" . _("X509v3 extensions") . ":</strong>\n";
 
             foreach ($certificate['extensions'] as $key => $value) {
                 if (is_array($value)) {
@@ -696,7 +697,7 @@ class Horde_Crypt_smime extends Horde_Crypt {
         }
 
         /* Certificate Details */
-        $text .= "<b>" . _("Certificate Details") . ":</b>\n";
+        $text .= "<strong>" . _("Certificate Details") . ":</strong>\n";
         $text .= sprintf("&nbsp;&nbsp;%s: %d\n", _("Version"), $certificate['version']);
         $text .= sprintf("&nbsp;&nbsp;%s: %d\n", _("Serial Number"), $certificate['serialNumber']);
 
@@ -1349,4 +1350,27 @@ class Horde_Crypt_smime extends Horde_Crypt {
         return $ob;
     }
 
+    /**
+     * The Content-Type parameters PHP's openssl_pkcs7_* functions return are
+     * deprecated.  Fix these headers to the correct ones (see RFC 2311).
+     *
+     * @access private
+     *
+     * @param string $text  The PKCS7 data.
+     * @param string $type  Is this 'message' or 'signature' data?
+     *
+     * @return string  The PKCS7 data with the correct Content-Type parameter.
+     */
+    function _fixContentType($text, $type)
+    {
+        if ($type == 'message') {
+            $from = 'application/x-pkcs7-mime';
+            $to = 'application/pkcs7-mime';
+        } else {
+            $from = 'application/x-pkcs7-signature';
+            $to = 'application/pkcs7-signature';
+        }
+        return str_replace('Content-Type: ' . $from, 'Content-Type: ' . $to, $text);
+    }
+
 }
