@@ -3,8 +3,8 @@
  * The Horde_Form_Action class provides an API for adding actions to
  * Horde_Form variables.
  *
- * $Horde: framework/Form/Form/Action.php,v 1.19.10.3 2005/06/07 06:05:49 selsky Exp $
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * $Horde: framework/Form/Form/Action.php,v 1.19.10.7 2007/01/02 13:54:17 jan Exp $
+ * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -45,7 +45,7 @@ class Horde_Form_Action {
 
     function _printJavaScriptStart()
     {
-        echo '<script language="JavaScript" type="text/javascript"><!--';
+        echo '<script type="text/javascript"><!--';
     }
 
     function _printJavaScriptEnd()
@@ -95,12 +95,15 @@ class Horde_Form_Action {
         } else {
             require_once 'Horde/Form/Action/' . $action . '.php';
         }
+
         $class = 'Horde_Form_Action_' . $action;
         if (class_exists($class)) {
-            return $ret = &new $class($params);
+            $instance = &new $class($params);
         } else {
-            return PEAR::raiseError('Class definition of ' . $class . ' not found.');
+            $instance = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
+
+        return $instance;
     }
 
     /**
