@@ -2,9 +2,9 @@
 /**
  * Line graph implementation for the Horde_Graph package.
  *
- * $Horde: framework/Graph/Graph/Plot/line.php,v 1.4.12.2 2005/04/21 11:26:41 selsky Exp $
+ * $Horde: framework/Graph/Graph/Plot/line.php,v 1.4.12.5 2007/01/02 13:54:20 jan Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -30,7 +30,7 @@ class Horde_Graph_Plot_line {
         }
     }
 
-    function draw()
+    function draw($minY = false)
     {
         $data = $this->_graph->_data['y'][$this->_dataset];
 
@@ -38,9 +38,20 @@ class Horde_Graph_Plot_line {
         $verts = array();
         for ($i = 0; $i < $count; $i++) {
             $x = $i;
-            $y = $data[$i];
-            $this->_graph->translate($x, $y);
-            $verts[] = array('x' => $x, 'y' => $y);
+            if ($minY !== false && $data[$i] < $minY) {
+                if (count($verts) == 1) {
+                    $this->_graph->img->circle($verts[0]['x'], $verts[0]['y'], $this->_width, $this->_color, $this->_color);
+                }
+                else {
+                    $this->_graph->img->polyline($verts, $this->_color, $this->_width);
+                }
+                $verts = array();
+            }
+            else {
+                $y = $data[$i];
+                $this->_graph->translate($x, $y);
+                $verts[] = array('x' => $x, 'y' => $y);
+            }
         }
 
         $this->_graph->img->polyline($verts, $this->_color, $this->_width);
