@@ -1,8 +1,6 @@
 <?php
 
 require_once dirname(__FILE__) . '/Message.php';
-require_once dirname(__FILE__) . '/Part.php';
-require_once HORDE_BASE . '/config/mime_drivers.php';
 
 /**
  * The name of the URL parameter that holds the MIME_Contents cache
@@ -10,15 +8,16 @@ require_once HORDE_BASE . '/config/mime_drivers.php';
  */
 define('MIME_CONTENTS_CACHE', 'mimecache');
 
-/* Attachment display constants. */
 /**
  * Display attachment information in list format.
  */
 define('MIME_CONTENTS_DISPLAY_TYPE_LIST', 0);
+
 /**
  * Display attachment information inline with attachment.
  */
 define('MIME_CONTENTS_DISPLAY_TYPE_INLINE', 1);
+
 /**
  * Display attachment information both in list format and inline with
  * attachment.
@@ -29,9 +28,9 @@ define('MIME_CONTENTS_DISPLAY_TYPE_BOTH', 2);
  * The MIME_Contents:: class contains functions related to handling the output
  * of MIME content.
  *
- * $Horde: framework/MIME/MIME/Contents.php,v 1.129.4.19 2005/09/15 16:34:47 jan Exp $
+ * $Horde: framework/MIME/MIME/Contents.php,v 1.129.4.37 2007/01/02 13:54:24 jan Exp $
  *
- * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (GPL). If you did not
  * receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -291,7 +290,9 @@ class MIME_Contents {
     {
         $msg = '';
 
-        foreach ($this->_atc as $key => $val) {
+        $akeys = array_keys($this->_atc);
+        natsort($akeys);
+        foreach ($akeys as $key) {
             $msg .= $this->_arrayToTableRow($this->_atc[$key]);
         }
 
@@ -313,13 +314,20 @@ class MIME_Contents {
         $partDisplayed = false;
 
         if ($oneframe) {
-            $msg .= '<style type="text/css">.onefr {padding: 1px; border: 1px dashed #00c; background-color: #eef; font-color: #006}</style>' . "\n";
+            $msg .= '<style type="text/css">.onefr {padding:1px; margin-bottom:1px; border:1px dashed #00c; background:#eef; color:#006}</style>' . "\n";
         }
 
+        // TODO: Temporary hack to display header info for a message with one
+        // MIME part that cannot be displayed inline.
+        if (!$msgCount || ($msgCount == 1 && !reset($this->_parts))) {
+            $this->setSummary($this->_message, 'part');
+            $msgCount = 1;
+        }
+            
         foreach ($this->_parts as $key => $value) {
             if (isset($this->_summary[$key]) &&
                 ((($msgCount == 1) && empty($value)) || ($msgCount > 1))) {
-                $msg .= '<tr><td align="left">';
+                $msg .= '<tr><td>';
                 if ($oneframe) {
                     $msg .= '<table class="onefr">';
                     $summary = $this->_summary[$key];
@@ -329,16 +337,16 @@ class MIME_Contents {
                     $msg .= '<table>';
                     $msg .= $this->_arrayToTableRow($this->_summary[$key]);
                 }
-                $msg .= '</table><div style="font-size: 1px;">&nbsp;<br /></div></td></tr>';
+                $msg .= '</table></td></tr>';
             }
             if (!empty($value)) {
-                $msg .= '<tr><td class="text" align="left">' . $value . '</td></tr>';
+                $msg .= '<tr><td class="text">' . $value . '</td></tr>';
                 $partDisplayed = true;
             }
         }
 
         if (!$partDisplayed) {
-            $msg .= '<tr><td class="text" align="left"><i>' . _("There are no parts that can be displayed inline.") . '</i></td></tr>';
+            $msg .= '<tr><td class="text" align="left"><em>' . _("There are no parts that can be displayed inline.") . '</em></td></tr>';
         }
 
         return $msg;
@@ -407,17 +415,22 @@ class MIME_Contents {
      */
     function canDisplayInline(&$mime_part)
     {
+        $viewer = &$this->getMIMEViewer($mime_part);
+        if (!$viewer) {
+            return false;
+        }
+
         /* First check: The MIME headers allow the part to be inlined.
          * However, if we are already in view mode, then we can skip this
          * check. */
         if (!$this->viewAsAttachment() &&
-            ($mime_part->getDisposition() != 'inline')) {
+            ($mime_part->getDisposition() != 'inline') &&
+            !$viewer->forceInlineView()) {
             return false;
         }
 
         /* Second check (save the most expensive for last):
          * Check to see if the driver is set to inline. */
-        $viewer = &$this->getMIMEViewer($mime_part);
         return (is_a($viewer, 'MIME_Viewer') && $viewer->canDisplayInline());
     }
 
@@ -432,7 +445,8 @@ class MIME_Contents {
     {
         /* Make sure we have a MIME_Part to process. */
         if (empty($mime_part)) {
-            return false;
+            $result = false;
+            return $result;
         }
 
         require_once dirname(__FILE__) . '/Viewer.php';
@@ -558,8 +572,8 @@ class MIME_Contents {
      *
      * @return string  The rendered data.
      */
-     function _renderMIMEPart(&$mime_part, $attachment = false)
-     {
+    function _renderMIMEPart(&$mime_part, $attachment = false)
+    {
         /* Get the MIME_Viewer object for this MIME part */
         $viewer = &$this->getMIMEViewer($mime_part);
         if (!is_a($viewer, 'MIME_Viewer')) {
@@ -583,12 +597,13 @@ class MIME_Contents {
                 $default_charset = String::upper(NLS::getCharset());
                 if ($charset_upper != $default_charset) {
                     $mime_part->setContents(String::convertCharset($mime_part->getContents(), $charset, $default_charset));
+                    $mime_part->setCharset($default_charset);
                     if ($default_charset != 'UTF-8') {
                         $status = array(
                             sprintf(_("This message was written in a character set (%s) other than your own."), htmlspecialchars($charset)),
                             sprintf(_("If it is not displayed correctly, %s to open it in a new window."), $this->linkViewJS($mime_part, 'view_attach', _("click here")))
                         );
-                        $msg = $this->formatStatusMsg($status, null, false) . '<p />' . $msg;
+                        $msg = $this->formatStatusMsg($status, null, false) . $msg;
                     }
                 }
             }
@@ -664,21 +679,22 @@ class MIME_Contents {
             }
         } else {
             $msg = $this->renderMIMEPart($mime_part);
-            if (!empty($msg)) {
-                $key = $this->_getMIMEKey($mime_part);
-                if (!$this->_attach822) {
-                    $this->_parts[$key] = $msg;
-                }
-                /* Some MIME_Viewers set the summary by themelves, so only
-                 * add to attachment/inline lists if nothing has been set
-                 * as of yet. */
-                if (!empty($key) && !$this->getIndex($key, 'summary')) {
-                    $this->setSummary($mime_part, 'part');
-                    if ($this->_attach822 &&
-                        (($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_LIST) ||
-                        ($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_BOTH))) {
-                        $this->setSummary($mime_part, 'attachment');
-                    }
+            $key = $this->_getMIMEKey($mime_part);
+            if (!$this->_attach822) {
+                $this->_parts[$key] = $msg;
+            }
+            /* Some MIME_Viewers set the summary by themelves, so only
+             * add to attachment/inline lists if nothing has been set
+             * as of yet. */
+            if ((($mime_part->getType() != 'multipart/mixed') ||
+                 !empty($msg)) &&
+                !empty($key) &&
+                !$this->getIndex($key, 'summary')) {
+                $this->setSummary($mime_part, 'part');
+                if ($this->_attach822 &&
+                    (($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_LIST) ||
+                    ($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_BOTH))) {
+                    $this->setSummary($mime_part, 'attachment');
                 }
             }
         }
@@ -794,7 +810,7 @@ class MIME_Contents {
         /* Number column. */
         if (($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_BOTH) &&
             !is_null($this->_summaryType)) {
-            $summary[] = '<a name="mime_contents_' . $this->_summaryType . '_' . $id . '" href="#mime_contents_' . (($this->_summaryType == 'attachment') ?'part' : 'attachment') . '_' . $id . '">' . $id . '</a>';
+            $summary[] = '<a id="mime_contents_' . $this->_summaryType . '_' . $id . '" href="#mime_contents_' . (($this->_summaryType == 'attachment') ?'part' : 'attachment') . '_' . $id . '">' . $id . '</a>';
         } else {
             $summary[] = $id;
         }
@@ -815,7 +831,7 @@ class MIME_Contents {
 
         /* MIME type column. */
         if ($guess) {
-            $summary[] = '<i>' . sprintf(_("View as %s"), $type) . '</i>';
+            $summary[] = '<em>' . sprintf(_("View as %s"), $type) . '</em>';
         } else {
             $summary[] = '[' . $type . ']';
         }
@@ -967,9 +983,9 @@ class MIME_Contents {
         }
 
         if ($widget) {
-            return Horde::widget('', $jstext, $css, null, "view('" . $url . "', '" . $id . "'); return false;", $text);
+            return Horde::widget('#', $jstext, $css, null, "view('" . $url . "', '" . $id . "'); return false;", $text);
         } else {
-            return Horde::link('#', $jstext, $css, null, "view('" . $url . "', '" . $id . "'); return false;", $text) . $text . '</a>';
+            return Horde::link('#', $jstext, $css, null, "view('" . $url . "', '" . $id . "'); return false;") . $text . '</a>';
         }
     }
 
@@ -1022,9 +1038,7 @@ class MIME_Contents {
             $text .= '</td></tr>' . "\n";
         }
 
-        $text .= '</table><div style="font-size: 1px;">&nbsp;<br /></div>';
-
-        return $text;
+        return $text . '</table>';
     }
 
     /**
@@ -1111,7 +1125,8 @@ class MIME_Contents {
 
         require_once 'Horde/SessionObjects.php';
         $cache = &Horde_SessionObjects::singleton();
-        return $ret = &$cache->query($cacheid);
+        $contents = &$cache->query($cacheid);
+        return $contents;
     }
 
     /**
