@@ -5,12 +5,18 @@ require_once dirname(__FILE__) . '/Part.php';
 require_once dirname(__FILE__) . '/../MIME.php';
 
 /**
- * $Horde: framework/MIME/MIME/Structure.php,v 1.87.10.5 2005/07/13 02:48:33 chuck Exp $
+ * $Horde: framework/MIME/MIME/Structure.php,v 1.87.10.21 2007/02/16 07:32:11 slusarz Exp $
  *
  * The MIME_Structure:: class provides methods for dealing with MIME mail.
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * The default character set to use for messages should be defined in the
+ * variable $GLOBALS['mime_structure']['default_charset'] (defaults to US-ASCII
+ * per RFC 2045).
+ *
+ * TODO: Convert to OO
+ *
+ * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -53,7 +59,7 @@ class MIME_Structure {
      *
      * @return MIME_Part  A MIME_Part object.
      */
-    function &_parse($body, $ref = null)
+    function &_parse($body, $ref = 0)
     {
         static $message, $multipart;
 
@@ -65,21 +71,17 @@ class MIME_Structure {
         $mime_part = &new MIME_Part();
 
         /* Top multiparts don't get their own line. */
-        if (is_null($ref) &&
+        if (empty($ref) &&
             (!isset($body->type) || ($body->type != $multipart))) {
             $ref = 1;
         }
 
         MIME_Structure::_setInfo($body, $mime_part, $ref);
 
-        /* Deal with multipart headers. */
-        if (is_null($ref)) {
-            $mime_part->setMIMEId(0);
-        } elseif ($body->subtype == 'RFC822') {
+        if (isset($body->type) && ($body->type == $message) &&
+            $body->ifsubtype && ($body->subtype == 'RFC822')) {
             $mime_part->setMIMEId($ref . '.0');
-        } elseif ((($body->subtype != 'MIXED') &&
-                   ($body->subtype != 'ALTERNATIVE')) ||
-                  ($body->type == $multipart)) {
+        } else {
             $mime_part->setMIMEId($ref);
         }
 
@@ -92,7 +94,7 @@ class MIME_Structure {
                     isset($sub_part->type) && ($sub_part->type == $multipart)) {
                     $sub_ref = $ref;
                 } else {
-                    $sub_ref = (is_null($ref)) ? $sub_id : $ref . '.' . $sub_id;
+                    $sub_ref = (empty($ref)) ? $sub_id : $ref . '.' . $sub_id;
                 }
                 $mime_part->addPart(MIME_Structure::_parse($sub_part, $sub_ref), $sub_id++);
             }
@@ -106,6 +108,9 @@ class MIME_Structure {
      * useful form and make sure that any parameters which are missing
      * are given default values.
      *
+     * To specify the default character set, define the global variable
+     * $GLOBALS['mime_strucutre']['default_charset'].
+     *
      * @access private
      *
      * @param stdClass $part  The original part info.
@@ -115,7 +120,7 @@ class MIME_Structure {
     function _setInfo($part, &$ob, $ref)
     {
         /* Store Content-type information. */
-        $primary_type = (isset($part->type)) ? $part->type : MIME::type('text');
+        $primary_type = (isset($part->type)) ? String::lower($part->type) : MIME::type('text');
         $sec_type = ($part->ifsubtype && $part->subtype) ? String::lower($part->subtype) : 'x-unknown';
         $ob->setType($primary_type . '/' . $sec_type);
 
@@ -141,15 +146,10 @@ class MIME_Structure {
         }
 
         /* Set the part's identification string, if available. */
-        if (!is_null($ref) && $part->ifid) {
-            $ob->setInformation('id', $part->id);
-        } else {
-            $ob->setInformation('id', false);
+        if (!empty($ref) && $part->ifid) {
+            $ob->setContentID($part->id);
         }
 
-        /* Set the default character set. */
-        $ob->setCharset(MIME_DEFAULT_CHARSET);
-
         /* Go through the content-type parameters, if any. */
         foreach (MIME_Structure::_getParameters($part, 1) as $key => $val) {
             if ($key == 'charset') {
@@ -159,6 +159,13 @@ class MIME_Structure {
             }
         }
 
+        /* Set the default character set. */
+        if (($ob->getPrimaryType() == 'text') &&
+            ($ob->getCharset() == 'us-ascii') &&
+            isset($GLOBALS['mime_structure']['default_charset'])) {
+            $ob->setCharset($GLOBALS['mime_structure']['default_charset']);
+        }
+
         /* Go through the disposition parameters, if any. */
         foreach (MIME_Structure::_getParameters($part, 2) as $key => $val) {
             $ob->setDispositionParameter($key, $val);
@@ -201,20 +208,35 @@ class MIME_Structure {
         $pexists = 'if' . $ptype;
 
         if ($part->$pexists) {
+            $attr_list = $rfc2231_list = array();
             foreach ($part->$ptype as $param) {
-                $param->value = str_replace("\t", ' ', $param->value);
-                $res = MIME::decodeRFC2231($param->attribute . '=' . $param->value);
-                if ($res) {
-                    $param->attribute = $res['attribute'];
-                    $param->value = $res['value'];
+                $param->value = str_replace(array("\t", '\"'), array(' ', '"'), $param->value);
+                /* Look for an asterisk in the attribute name.  If found we
+                 * know we have RFC 2231 information. */
+                $pos = strpos($param->attribute, '*');
+                if ($pos) {
+                    $attr = substr($param->attribute, 0, $pos);
+                    $rfc2231_list[$attr][] = $param->attribute . '=' . $param->value;
+                } else {
+                    $attr_list[$param->attribute] = $param->value;
                 }
-                $field = String::lower($param->attribute);
+            }
+
+            foreach ($rfc2231_list as $val) {
+                $res = MIME::decodeRFC2231(implode(' ', $val));
+                if ($res) {
+                    $attr_list[$res['attribute']] = $res['value'];
+                }
+            }
+
+            foreach ($attr_list as $attr => $val) {
+                $field = String::lower($attr);
                 if ($field == 'type') {
-                    if (($type = MIME::type($param->value))) {
+                    if (($type = MIME::type($val))) {
                         $param_list['type'] = $type;
                     }
                 } else {
-                    $param_list[$field] = $param->value;
+                    $param_list[$field] = $val;
                 }
             }
         }
@@ -227,8 +249,9 @@ class MIME_Structure {
      *
      * @access private
      *
-     * @param array &$parts  TODO
-     * @param array $info    TODO
+     * @param array &$parts  The list of parts contained within the multipart
+     *                       object.
+     * @param array $info    Information about the multipart structure.
      */
     function _addMultipartInfo(&$parts, $info = array())
     {
@@ -245,7 +268,7 @@ class MIME_Structure {
             }
             if (isset($info['related'])) {
                 $ptr->setInformation('related_part', $info['related']->getMIMEId());
-                if (($id = $ptr->getInformation('id'))) {
+                if ($id = $ptr->getContentID()) {
                     $info['related']->addCID(array($ptr->getMIMEId() => $id));
                 }
             }
@@ -292,13 +315,15 @@ class MIME_Structure {
 
         $mimeDecode = &new Mail_mimeDecode($text, MIME_PART_EOL);
         if (!($structure = $mimeDecode->decode($decode_args))) {
-            return false;
+            $message = false;
+        } else {
+            /* Put the object into imap_parsestructure() form. */
+            MIME_Structure::_convertMimeDecodeData($structure);
+
+            $message = MIME_Structure::parse($structure);
         }
 
-        /* Put the object into imap_parsestructure() form. */
-        MIME_Structure::_convertMimeDecodeData($structure);
-
-        return $ret = &MIME_Structure::parse($structure);
+        return $message;
     }
 
     /**
