@@ -22,11 +22,11 @@
  *
  * The following tags are NOT implemented:
  * <indentright>, <outdent>, <outdentright>, <samepage>, <iso-8859-X>,
- * <us-ascii>, 
+ * <us-ascii>,
  *
- * $Horde: framework/MIME/MIME/Viewer/richtext.php,v 1.4.10.5 2005/07/03 05:04:10 selsky Exp $
+ * $Horde: framework/MIME/MIME/Viewer/richtext.php,v 1.4.10.8 2007/01/02 13:54:26 jan Exp $
  *
- * Copyright 2004-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2004-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -73,7 +73,7 @@ class MIME_Viewer_richtext extends MIME_Viewer {
 
         /* <lt> becomes a '<'. CRLF becomes a SPACE. */
         if ($has_str_ireplace) {
-            $text = str_ireplace(array('<lt>', "\r\n"), array('&lt;', ' '));
+            $text = str_ireplace(array('<lt>', "\r\n"), array('&lt;', ' '), $text);
         } else {
             $text = preg_replace(array('/<lt>/i', "/\r\n/"), array('&lt;', ' '), $text);
         }
@@ -84,7 +84,7 @@ class MIME_Viewer_richtext extends MIME_Viewer {
         /* <nl> becomes a newline (<br />);
          * <np> becomes a paragraph break (<p />). */
         if ($has_str_ireplace) {
-            $text = str_ireplace(array('&lt;nl&gt;', '&lt;np&gt;'), array('<br />', '<p />'));
+            $text = str_ireplace(array('&lt;nl&gt;', '&lt;np&gt;'), array('<br />', '<p />'), $text);
         } else {
             $text = preg_replace(array('/(?<!&lt;)&lt;nl.*&gt;/Uis', '/(?<!&lt;)&lt;np.*&gt;/Uis'), array('<br />', '<p />'), $text);
         }
@@ -130,7 +130,7 @@ class MIME_Viewer_richtext extends MIME_Viewer {
     /**
      * Return the MIME content type of the rendered content.
      *
-     * @return string  The content type of the output. 
+     * @return string  The content type of the output.
      */
     function getType()
     {
