@@ -2,10 +2,10 @@
 /**
  * The MIME_Viewer_tgz class renders out plain or gzipped tarballs in HTML.
  *
- * $Horde: framework/MIME/MIME/Viewer/tgz.php,v 1.37.10.9.2.1 2005/11/22 09:00:39 jan Exp $
+ * $Horde: framework/MIME/MIME/Viewer/tgz.php,v 1.37.10.14 2007/01/02 13:54:26 jan Exp $
  *
- * Copyright 1999-2005 Anil Madhavapeddy <anil@recoil.org>
- * Copyright 2002-2005 Michael Cochrane <mike@graftonhall.co.nz>
+ * Copyright 1999-2007 Anil Madhavapeddy <anil@recoil.org>
+ * Copyright 2002-2007 Michael Cochrane <mike@graftonhall.co.nz>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -63,7 +63,7 @@ class MIME_Viewer_tgz extends MIME_Viewer {
         /* Obtain the list of files/data in the tar file. */
         $tar = &Horde_Compress::singleton('tar');
 
-        $tarData = &$tar->decompress($contents);
+        $tarData = $tar->decompress($contents);
         if (is_a($tarData, 'PEAR_Error')) {
             return '<pre>' . $tarData->getMessage() . '</pre>';
         }
@@ -72,7 +72,7 @@ class MIME_Viewer_tgz extends MIME_Viewer {
 
         include_once 'Horde/Text.php';
 
-        $text  = '<b>' . htmlspecialchars(sprintf(_("Contents of \"%s\""), $this->mime_part->getName())) . ':</b>' . "\n";
+        $text  = '<strong>' . htmlspecialchars(sprintf(_("Contents of \"%s\""), $this->mime_part->getName())) . ':</strong>' . "\n";
         $text .= '<table><tr><td align="left"><tt><span class="fixed">';
         $text .= Text::htmlAllSpaces(_("Archive Name") . ':  ' . $this->mime_part->getName()) . "\n";
         $text .= Text::htmlAllSpaces(_("Archive File Size") . ': ' . strlen($contents) . ' bytes') . "\n";
@@ -104,9 +104,9 @@ class MIME_Viewer_tgz extends MIME_Viewer {
 
     /**
      * Return the content-type
-     * 
+     *
      * @return string  The content-type of the output.
-     */ 
+     */
     function getType()
     {
         return 'text/html; charset=' . NLS::getCharset();
