@@ -64,9 +64,9 @@ $GLOBALS['intervals'] = &$intervals;
  * The Maintenance:: class provides a set of methods for dealing with
  * maintenance operations run upon login to Horde applications.
  *
- * $Horde: framework/Maintenance/Maintenance.php,v 1.52.10.10 2005/07/03 05:52:00 selsky Exp $
+ * $Horde: framework/Maintenance/Maintenance.php,v 1.52.10.16 2007/02/09 12:32:10 jan Exp $
  *
- * Copyright 2001-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2001-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -135,10 +135,12 @@ class Maintenance {
         include_once $registry->get('fileroot', $module) . '/lib/Maintenance/' . $module . '.php';
         $class = 'Maintenance_' . $module;
         if (class_exists($class)) {
-            return $ret = &new $class($params);
+            $maintenance = &new $class($params);
         } else {
-            return false;
+            $maintenance = false;
         }
+
+        return $maintenance;
     }
 
     /**
@@ -159,20 +161,19 @@ class Maintenance {
     }
 
     /**
-     * Do maintenance operations needed for this login.  This function
-     * will generate the list of tasks to perform during this login
-     * and will redirect to the maintenance page if necessary.  This
-     * is the function that should be called from the application upon
-     * login.
+     * Do maintenance operations needed for this login.
+     *
+     * This function will generate the list of tasks to perform during this
+     * login and will redirect to the maintenance page if necessary.  This is
+     * the function that should be called from the application upon login.
      */
     function runMaintenance()
     {
-        /* Check to see if we are finished with maintenance
-         * operations. */
+        /* Check to see if we are finished with maintenance operations. */
         if (!Util::getFormData(MAINTENANCE_DONE_PARAM)) {
             /* Determine if we should redirect to the maintenance page. */
             if ($this->_needMaintenancePage() !== null) {
-                header('Location: ' . $this->_getMaintenanceURL(), true);
+                header('Location: ' . $this->_getMaintenanceURL());
                 exit;
             }
         }
@@ -183,8 +184,10 @@ class Maintenance {
 
     /**
      * Do the necessary maintenance tasks for this loading of the maintenance
-     * page.  This is the function that is called from the maintenance page
-     * every time it is loaded.
+     * page.
+     *
+     * This is the function that is called from the maintenance page every
+     * time it is loaded.
      *
      * @return integer  The display required for the maintenance page.
      */
@@ -194,19 +197,22 @@ class Maintenance {
         $this->_doMaintenanceTasks();
 
         /* Get the list of tasks we need to display to the user. */
-        $task_no = $this->_needMaintenancePage();
+        $tasks_page = $this->_needMaintenancePage();
         $tasks = $this->_tasklist->getList();
 
         /* Remove 'newflag' from first task. */
         if (!$this->_tasklist->processed(true)) {
-            if (count($tasks)) {
-                reset($tasks);
-                $this->_tasklist->setNewPage(key($tasks), false);
+            if (count($tasks_page)) {
+                $this->_tasklist->setNewPage($tasks_page[0], false);
             }
         }
 
-        if (!is_null($task_no)) {
-            $tasks = array_slice($tasks, 0, $task_no + 1);
+        if (!is_null($tasks_page)) {
+            foreach (array_keys($tasks) as $key) {
+                if (!in_array($key, $tasks_page)) {
+                    unset($tasks[$key]);
+                }
+            }
         }
 
         if (count($tasks)) {
@@ -324,8 +330,9 @@ class Maintenance {
         /* Go through each item in $maint_tasks and determine if we need to
            run it during this maintenance run. */
         foreach ($this->maint_tasks as $key => $val) {
-            /* Skip item if it already appears in the tasks list. */
-            if ($this->_tasklist->inList($key)) {
+            /* Skip item if it already appears in the tasks list or task is
+             * not set in the preferences. */
+            if ($this->_tasklist->inList($key) || !$prefs->getValue($key)) {
                 continue;
             }
 
@@ -374,10 +381,11 @@ class Maintenance {
             elseif ($val == MAINTENANCE_EVERY) {
                 $addTask = true;
             }
-            /* Skip the task if any of the following:
-               + This task does not need to be run in this login
-               + This task is not set in the preferences */
-            if (!$addTask || !$prefs->getValue($key)) continue;
+            /* Skip the task if this task does not need to be run in this
+             * login. */
+            if (!$addTask) {
+                continue;
+            }
 
             /* Load the task module now. */
             $mod = &$this->_loadModule($key);
@@ -385,11 +393,7 @@ class Maintenance {
             /* Determine if this task has already been confirmed/set via some
                sort of admin setting. Also, if the user/admin has set the
                'confirm_maintenance' flag, skip confirmation. */
-            $confirmed = false;
-            if ($prefs->isLocked($key) ||
-                !$prefs->getValue('confirm_maintenance')) {
-                $confirmed = true;
-            }
+            $confirmed = $prefs->isLocked($key) || !$prefs->getValue('confirm_maintenance');
 
             /* Add the task to the tasklist. */
             $this->_tasklist->addTask($key, $confirmed, $mod->getDisplayType());
@@ -512,9 +516,8 @@ class Maintenance {
             $this->_tasklist->removeTask($key);
         }
 
-        /* If we've successfully completed every task in the list (or
-         * skipped it), record now as the last time maintenance was
-         * run. */
+        /* If we've successfully completed every task in the list (or skipped
+         * it), record now as the last time maintenance was run. */
         if (!count($this->_tasklist->getList())) {
             $GLOBALS['prefs']->setValue('last_maintenance', time());
         }
@@ -525,23 +528,27 @@ class Maintenance {
      *
      * @access private
      *
-     * @return integer  The key of the task that requires the maintenance
-     *                  page. Returns null if the maintenance page is no
-     *                  longer needed.
+     * @return array  The list of tasks that require the maintenance page or
+     *                null if the maintenance page is no longer needed.
      */
     function _needMaintenancePage()
     {
-        $i = 0;
-        foreach ($this->_tasklist->getList() as $key => $val) {
+        $tasks = array();
+        $tasklist = $this->_tasklist->getList();
+        while (list($key, $val) = each($tasklist)) {
             if ($val['newpage']) {
-                return $i;
+                $tasks[] = $key;
+                while ((list($key, $val) = each($tasklist)) &&
+                       !$val['newpage']) {
+                    $tasks[] = $key;
+                }
+                return $tasks;
             }
             if (!empty($val['process']) &&
                 !Util::getFormData('domaintenance')) {
                 $this->_tasklist->setNewPage($key, true);
-                return $i;
+                return $this->_needMaintenancePage();
             }
-            $i++;
         }
 
         return null;
@@ -553,7 +560,7 @@ class Maintenance {
  * The Maintenance_Tasklist:: class is used to store the list of maintenance
  * tasks that need to be run during this login.
  *
- * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -622,7 +629,7 @@ class Maintenance_Tasklist {
         global $registry;
 
         $this->_module = $registry->getApp();
-        $this->_target = Horde::selfUrl(true);
+        $this->_target = Horde::selfUrl(true, true, true);
     }
 
     /**
@@ -642,7 +649,7 @@ class Maintenance_Tasklist {
             ($display == MAINTENANCE_OUTPUT_NOTICE)) {
             $this->_tasks[$key]['newpage'] = true;
             $this->_addFlag = false;
-        } elseif (($confirmed == false) && (!$this->_addFlag)) {
+        } elseif (!$confirmed && !$this->_addFlag) {
             $this->_tasks[$key]['newpage'] = true;
             $this->_addFlag = true;
         } else {
@@ -803,7 +810,7 @@ class Maintenance_Tasklist {
  *     text. doMaintenance() will ALWAYS be run.
  *     * This style will be displayed on its own, separate maintenance page. *
  *
- * Copyright 2001-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2001-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
