@@ -1,7 +1,5 @@
 <?php
 
-// Preference constants (bitmasks)
-
 /**
  * Preference is administratively locked.
  */
@@ -22,8 +20,6 @@ define('_PREF_DIRTY', 4);
  */
 define('_PREF_DEFAULT', 8);
 
-require_once 'Horde/String.php';
-
 /**
  * The Prefs:: class provides a common abstracted interface into the
  * various preferences storage mediums.  It also includes all of the
@@ -52,9 +48,9 @@ require_once 'Horde/String.php';
  *     'help'   => *Name of the entry in the XML help file*
  * );
  *
- * $Horde: framework/Prefs/Prefs.php,v 1.137.10.11 2005/07/23 22:38:17 slusarz Exp $
+ * $Horde: framework/Prefs/Prefs.php,v 1.137.10.29 2007/01/02 13:54:35 jan Exp $
  *
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 1999-2007 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -139,10 +135,12 @@ class Prefs {
     }
 
     /**
-     * Updates the session-based preferences cache (if available) with the
-     * current set of preferences.
+     * Updates the session-based preferences cache (if available).
+     *
+     * @param string $pref  The preference to update.  If empty, will update
+     *                      the entire cache with the current set of prefs.
      */
-    function cacheUpdate()
+    function cacheUpdate($pref = null)
     {
         /* Return immediately if caching is disabled. */
         if (!$this->_caching) {
@@ -150,9 +148,15 @@ class Prefs {
         }
 
         if (isset($_SESSION['prefs_cache'])) {
+            if (is_null($pref)) {
+                $prefs = $this->_prefs;
+            } else {
+                $prefs = array($pref => $this->_prefs[$pref]);
+            }
+
             /* Place each preference in the cache according to its
              * scope. */
-            foreach ($this->_prefs as $name => $pref) {
+            foreach ($prefs as $name => $pref) {
                 $_SESSION['prefs_cache'][$this->getScope($name)][$name] = $pref;
             }
         }
@@ -172,18 +176,10 @@ class Prefs {
         }
 
         if (isset($_SESSION['prefs_cache']['horde']) &&
-            !empty($_SESSION['prefs_cache']['_filled'][$this->_scope]) &&
             isset($_SESSION['prefs_cache'][$this->_scope])) {
 
             /* Restore global preferences. */
-            foreach ($_SESSION['prefs_cache']['horde'] as $name => $pref) {
-                $this->_prefs[$name] = $pref;
-            }
-
-            /* Restore application-specific preferences. */
-            foreach ($_SESSION['prefs_cache'][$this->_scope] as $name => $pref) {
-                $this->_prefs[$name] = $pref;
-            }
+            $this->_prefs = array_merge($this->_prefs, $_SESSION['prefs_cache']['horde'], $_SESSION['prefs_cache'][$this->_scope]);
 
             return true;
         }
@@ -213,7 +209,9 @@ class Prefs {
     function remove($pref)
     {
         if (is_array($this->_prefs)) {
+            $scope = $this->getScope($pref);
             unset($this->_prefs[$pref]);
+            unset($_SESSION['prefs_cache'][$scope][$pref]);
         }
     }
 
@@ -240,6 +238,11 @@ class Prefs {
         return $this->_setValue($pref, $val, true, $convert);
     }
 
+    function __set($name, $value)
+    {
+        return $this->setValue($name, $value);
+    }
+
     /**
      * Sets the given preferences ($pref) to the specified value
      * ($val), whether or not the preference is user-modifiable, unset
@@ -299,6 +302,8 @@ class Prefs {
             $this->setDirty($pref, true);
         }
 
+        $this->cacheUpdate($pref);
+
         return true;
     }
 
@@ -319,11 +324,28 @@ class Prefs {
             $charset = NLS::getCharset();
         }
 
-        return (isset($this->_prefs[$pref]['v'])) ?
-            ($convert ?
-             $this->convertFromDriver($this->_prefs[$pref]['v'], $charset) :
-             $this->_prefs[$pref]['v']) :
-            null;
+        if (isset($this->_prefs[$pref]['v'])) {
+            if ($convert) {
+                if ($this->isDefault($pref)) {
+                    /* Default values have the current UI charset. */
+                    $value = String::convertCharset($this->_prefs[$pref]['v'], NLS::getCharset(), $charset);
+                } else {
+                    /* Stored values have the backend charset. */
+                    $value = $this->convertFromDriver($this->_prefs[$pref]['v'], $charset);
+                }
+            } else {
+                $value = $this->_prefs[$pref]['v'];
+            }
+        } else {
+            $value = null;
+        }
+
+        return $value;
+    }
+
+    function __get($name)
+    {
+        return $this->getValue($name);
     }
 
     /**
@@ -400,8 +422,6 @@ class Prefs {
      *
      * @param string $pref   The name of the preference to modify.
      * @param boolean $bool  The new boolean value for the "default" bit.
-     *
-     * @since Horde 2.1
      */
     function setDefault($pref, $bool)
     {
@@ -430,8 +450,6 @@ class Prefs {
      *
      * @return boolean  True if the preference is the application default
      *                  value.
-     *
-     * @since Horde 2.1
      */
     function isDefault($pref)
     {
@@ -524,9 +542,7 @@ class Prefs {
     }
 
     /**
-     * This method should be overridden by a subclass
-     * implementation. It's here to retain code integrity in the case
-     * that no subclass is loaded ($driver == 'none').
+     * Retrieves the default preferences.
      */
     function retrieve()
     {
@@ -538,15 +554,11 @@ class Prefs {
     }
 
     /**
-     * This is an abstract method that should be overridden by a
-     * subclass implementation. It's here to retain code integrity in
-     * the case that no subclass is loaded ($driver == 'none').
-     *
-     * This function will be run at the end of every request as a
-     * shutdown function (registered by the Prefs:: constructor).  All
-     * prefs with the dirty bit set will be saved to the storage
-     * backend at this time; thus, there is no need to manually call
-     * $prefs->store() every time a preference is changed.
+     * This function will be run at the end of every request as a shutdown
+     * function (registered by the Prefs:: constructor).  All prefs with the
+     * dirty bit set will be saved to the storage backend at this time; thus,
+     * there is no need to manually call $prefs->store() every time a
+     * preference is changed.
      *
      * @abstract
      */
@@ -565,13 +577,8 @@ class Prefs {
     {
         /* Remove this scope from the preferences cache, if it
            exists. */
-        if (isset($_SESSION['prefs_cache'])) {
-            if (isset($_SESSION['prefs_cache'][$this->_scope])) {
-                unset($_SESSION['prefs_cache'][$this->_scope]);
-            }
-            if (isset($_SESSION['prefs_cache']['_filled'][$this->_scope])) {
-                unset($_SESSION['prefs_cache']['_filled'][$this->_scope]);
-            }
+        if (isset($_SESSION['prefs_cache'][$this->_scope])) {
+            unset($_SESSION['prefs_cache'][$this->_scope]);
         }
 
         /* Perform a Horde-wide cleanup? */
@@ -705,7 +712,7 @@ class Prefs {
         }
 
         global $registry;
-        include $registry->get('fileroot', 'horde') . '/config/hooks.php';
+        include_once $registry->get('fileroot', 'horde') . '/config/hooks.php';
         foreach ($this->_hooks as $pref) {
             if ($this->isLocked($pref) ||
                 !$this->getValue($pref) ||
@@ -744,29 +751,25 @@ class Prefs {
             $driver = $driver[1];
         }
 
-        /* Attempt to register (cache) the $prefs hash in session
-         * storage. */
+        /* Attempt to register (cache) the $prefs hash in session storage. */
         if ($caching) {
             if (!isset($_SESSION['prefs_cache'])) {
                 $_SESSION['prefs_cache'] = array();
-                $_SESSION['prefs_cache']['_filled'] = array();
             }
         }
 
         $driver = basename($driver);
         if (empty($driver) || $driver == 'none') {
-            $prefs = &new Prefs();
-            $prefs->_scope = $scope;
-            return $prefs;
+            $driver = 'session';
         }
 
         if (is_null($params)) {
             $params = Horde::getDriverConfig('prefs', $driver);
         }
 
-        /* If $params['user_hook'] is defined, use it to retrieve the
-           value to use for the username ($this->_user). Otherwise,
-           just use the value passed in the $user parameter. */
+        /* If $params['user_hook'] is defined, use it to retrieve the value to
+         * use for the username ($this->_user). Otherwise, just use the value
+         * passed in the $user parameter. */
         if (!empty($params['user_hook']) &&
             function_exists($params['user_hook'])) {
             $user = call_user_func($params['user_hook'], $user);
@@ -774,17 +777,20 @@ class Prefs {
 
         if (!empty($app)) {
             require_once $GLOBALS['registry']->get('fileroot', $app) . '/lib/Prefs/' . $driver . '.php';
-        } elseif (@file_exists(dirname(__FILE__) . '/Prefs/' . $driver . '.php')) {
+        } elseif (file_exists(dirname(__FILE__) . '/Prefs/' . $driver . '.php')) {
             require_once dirname(__FILE__) . '/Prefs/' . $driver . '.php';
         } else {
-            @include_once 'Horde/Prefs/' . $driver . '.php';
+            include_once 'Horde/Prefs/' . $driver . '.php';
         }
+
         $class = 'Prefs_' . $driver;
         if (class_exists($class)) {
-            return $ret = &new $class($user, $password, $scope, $params, $caching);
+            $prefs = &new $class($user, $password, $scope, $params, $caching);
         } else {
-            return PEAR::raiseError('Class definition of ' . $class . ' not found.');
+            $prefs = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
+
+        return $prefs;
     }
 
     /**
@@ -814,11 +820,7 @@ class Prefs {
     function &singleton($driver, $scope = 'horde', $user = '', $password = '',
                         $params = null, $caching = true)
     {
-        static $instances;
-
-        if (!isset($instances)) {
-            $instances = array();
-        }
+        static $instances = array();
 
         if (is_null($params)) {
             $params = Horde::getDriverConfig('prefs', $driver);
