@@ -3,9 +3,9 @@
  * Class for auto-generating the preferences user interface and
  * processing the forms.
  *
- * $Horde: framework/Prefs/Prefs/UI.php,v 1.63.2.10.2.1 2005/12/27 17:02:01 chuck Exp $
+ * $Horde: framework/Prefs/Prefs/UI.php,v 1.63.2.16 2007/01/02 13:54:35 jan Exp $
  *
- * Copyright 2001-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2001-2007 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -187,12 +187,6 @@ class Prefs_UI {
             array_unshift($GLOBALS['language_options'], _("Default"));
         }
 
-        /* If there's only one prefGroup, just show it. */
-        if (empty($group) && count($prefGroups) == 1) {
-            $group = array_keys($prefGroups);
-            $group = array_pop($group);
-        }
-
         if (!empty($group) && Prefs_UI::groupIsEditable($group)) {
             foreach ($prefGroups[$group]['members'] as $pref) {
                 if (!$prefs->isLocked($pref)) {
@@ -246,9 +240,12 @@ class Prefs_UI {
      */
     function generateHeader($group = null)
     {
-        global $registry, $prefGroups, $app, $perms;
+        global $registry, $prefGroups, $app, $perms, $prefs, $notification;
 
         $title = _("User Options");
+        if ($group == 'identities' && !$prefs->isLocked('default_identity')) {
+            $notification->push('newChoice()', 'javascript');
+        }
         require $registry->get('templates', $app) . '/common-header.inc';
         if (is_callable(array($app, 'getMenu'))) {
             $menu = call_user_func(array($app, 'getMenu'));
@@ -269,8 +266,8 @@ class Prefs_UI {
         /* Get list of accessible applications. */
         $apps = array();
         foreach ($registry->applications as $application => $params) {
-            // Make sure the app is installed
-            if (!file_exists($registry->get('fileroot', $application))) {
+            // Make sure the app is installed and has a prefs file.
+            if (!file_exists($registry->get('fileroot', $application) . '/config/prefs.php')) {
                 continue;
             }
 
@@ -309,10 +306,9 @@ class Prefs_UI {
      * Generate the content of the title bar navigation cell (previous | next
      * option group).
      *
-     * @param          string $group        Current option group.
-     * @param optional string $attributes   The optional <td> attributes.
+     * @param string $group  Current option group.
      */
-    function generateNavigationCell($group, $attributes = 'class="header" align="right"')
+    function generateNavigationCell($group)
     {
         global $prefGroups, $registry, $app;
 
@@ -354,7 +350,7 @@ class Prefs_UI {
             return;
         }
 
-        echo "<td $attributes><span class='smallheader'>";
+        echo '<ul><li>';
         if (!empty($prefGroups[$previous]['url'])) {
             echo Horde::link(Horde::applicationUrl($prefGroups[$previous]['url']),
                              _("Previous options"));
@@ -374,7 +370,7 @@ class Prefs_UI {
                              _("Next options"));
             echo $prefGroups[$next]['label'] . ' &gt;&gt;';
         }
-        echo '</a></span></td>';
+        echo '</a></li></ul>';
     }
 
     /**
