@@ -7,9 +7,9 @@ require_once 'Horde/Prefs/ldap.php';
  * Prefs_ldap LDAP authentication object, and simply provides parameters to it
  * based on the global Kolab configuration.
  *
- * $Horde: framework/Prefs/Prefs/kolab.php,v 1.1.10.8 2006/06/21 20:09:07 jan Exp $
+ * $Horde: framework/Prefs/Prefs/kolab.php,v 1.1.10.10 2007/01/02 13:54:35 jan Exp $
  *
- * Copyright 2004-2006 Stuart Binge <s.binge@codefusion.co.za>
+ * Copyright 2004-2007 Stuart Binge <s.binge@codefusion.co.za>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -32,14 +32,14 @@ class Prefs_kolab extends Prefs_ldap {
     function Prefs_kolab($user, $password, $scope = '',
                          $params = array(), $caching = false)
     {
-        $params['hostspec'] = $GLOBALS['conf']['kolab']['ldap']['server'];
-        $params['port'] = $GLOBALS['conf']['kolab']['ldap']['port'];
-        $params['version'] = '3';
-        $params['basedn'] = $GLOBALS['conf']['kolab']['ldap']['basedn'];
-        $params['fetchdn'] = true;
-        $params['searchdn'] = $GLOBALS['conf']['kolab']['ldap']['phpdn'];
-        $params['searchpass'] = $GLOBALS['conf']['kolab']['ldap']['phppw'];
-        $params['uid'] = array('mail', 'uid');
+        $params = array('hostspec' => $GLOBALS['conf']['kolab']['ldap']['server'],
+                        'port' => $GLOBALS['conf']['kolab']['ldap']['port'],
+                        'version' => '3',
+                        'basedn' => $GLOBALS['conf']['kolab']['ldap']['basedn'],
+                        'fetchdn' => true,
+                        'searchdn' => $GLOBALS['conf']['kolab']['ldap']['phpdn'],
+                        'searchpass' => $GLOBALS['conf']['kolab']['ldap']['phppw'],
+                        'uid' => array('mail', 'uid'));
 
         parent::Prefs_ldap($user, $password, $scope, $params, $caching);
     }
