@@ -11,9 +11,9 @@ require_once 'Net/IMSP.php';
  *   'server'    The hostname of the IMSP server.
  *   'port'      The port of the IMSP server.</pre>
  *
- * $Horde: framework/Net_IMSP/IMSP/Auth.php,v 1.8.10.7 2005/07/03 05:02:04 selsky Exp $
+ * $Horde: framework/Net_IMSP/IMSP/Auth.php,v 1.8.10.12 2007/01/02 13:54:28 jan Exp $
  *
- * Copyright 2003-2005 Michael Rubinsky <mrubinsk@horde.org>
+ * Copyright 2003-2007 Michael Rubinsky <mrubinsk@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -34,6 +34,7 @@ class Net_IMSP_Auth {
      *
      * @param array $params         Parameters for Net_IMSP
      * @param boolean $login        Should we remain logged in after auth?
+     *
      * @return mixed                Returns a Net_IMSP object connected to
      *                              the IMSP server if login is true and
      *                              successful.  Returns boolean true if
@@ -60,18 +61,19 @@ class Net_IMSP_Auth {
      * Private authentication function. Provides actual authentication
      * code.
      *
-     * @abstract
      * @access private
      * @param  array   $params      Parameters for Net_IMSP_Auth driver.
+     *
      * @return mixed                Returns Net_IMSP object connected to server
      *                              if successful, PEAR_Error on failure.
+     * @abstract
      */
     function &_authenticate($params)
     {
 
     }
+
     /**
-     * Attempts to return a concrete Net_IMSP_Auth instance based on $driver
      * Returns the type of this driver.
      *
      * @abstract
@@ -92,9 +94,11 @@ class Net_IMSP_Auth {
     }
 
     /**
+     * Attempts to return a concrete Net_IMSP_Auth instance based on $driver
      * Must be called as &Net_IMSP_Auth::factory()
      *
      * @param  string $driver Type of Net_IMSP_Auth subclass to return.
+     *
      * @return mixed  The created Net_IMSP_Auth subclass or PEAR_Error.
      */
     function &factory($driver)
@@ -102,7 +106,8 @@ class Net_IMSP_Auth {
         $driver = basename($driver);
 
         if (empty($driver) || (strcmp($driver, 'none') == 0)) {
-            return $ret = &new IMSP_Auth();
+            $auth = &new IMSP_Auth();
+            return $auth;
         }
 
         if (file_exists(dirname(__FILE__) . '/Auth/' . $driver . '.php')) {
@@ -112,9 +117,10 @@ class Net_IMSP_Auth {
         $class = 'Net_IMSP_Auth_' . $driver;
 
         if (class_exists($class)) {
-            return $ret = &new $class();
+            $auth = &new $class();
+            return $auth;
         } else {
-           Horde::fatal(PEAR::raiseError(sprintf(_("Unable to load the definition of %s."), $class)), __FILE__, __LINE__);
+            Horde::fatal(PEAR::raiseError(sprintf(_("Unable to load the definition of %s."), $class)), __FILE__, __LINE__);
         }
     }
 
@@ -125,12 +131,22 @@ class Net_IMSP_Auth {
      * Must be called like: $var = &Net_IMSP_Auth::singleton('driver_type');
      *
      * @param  string $driver Type of IMSP_Auth subclass to return.
+     *
      * @return object Reference to IMSP_Auth subclass.
      */
     function &singleton($driver)
     {
         static $instances;
-
+        /* Check for any imtest driver instances and kill them.
+           Otherwise, the socket will hang between requests from
+           seperate drivers (an Auth request and an Options request).*/
+        if (is_array($instances)) {
+            foreach ($instances as $obj) {
+                if ($obj->getDriverType() == 'imtest') {
+                    $obj->logout();
+                }
+            }
+        }
         if (!isset($instances)) {
             $instances = array();
         }
