@@ -46,15 +46,14 @@ define('VFS_FOLDER', 2);
  *    mssql.textsize = 0 ; zero to pass through
  * </pre>
  *
- * $Horde: framework/VFS/VFS/sql.php,v 1.85.4.16 2005/08/24 14:04:39 selsky Exp $
+ * $Horde: framework/VFS/VFS/sql.php,v 1.85.4.29 2007/01/02 13:54:48 jan Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @since   Horde 2.2
  * @package VFS
  */
 class VFS_sql extends VFS {
@@ -66,6 +65,70 @@ class VFS_sql extends VFS {
      */
     var $_db = false;
 
+    /**
+     * Retrieves the filesize from the VFS.
+     *
+     * @param string $path  The pathname to the file.
+     * @param string $name  The filename to retrieve.
+     *
+     * @return int The file size.
+     */
+    function size($path, $name)
+    {
+        $conn = $this->_connect();
+        if (is_a($conn, 'PEAR_Error')) {
+            return $conn;
+        }
+
+        $length_op = $this->_getFileSizeOp();
+        $sql = sprintf(
+            'SELECT %s(vfs_data) FROM %s WHERE vfs_path = ? AND vfs_name = ?',
+            $length_op,
+            $this->_params['table']
+        );
+        $values = array($this->_convertPath($path), $name);
+        $this->log($sql, PEAR_LOG_DEBUG);
+        $size = $this->_db->getOne($sql, $values);
+
+        if (is_null($size)) {
+            return PEAR::raiseError(sprintf(_("Unable to check file size of \"%s/%s\"."), $path, $name));
+        }
+
+        return $size;
+    }
+
+    /**
+     * Returns the size of a file.
+     *
+     * @access public
+     *
+     * @param string $path  The path of the file.
+     * @param string $name  The filename.
+     *
+     * @return integer  The size of the folder in bytes or PEAR_Error on
+     *                  failure.
+     */
+    function getFolderSize($path = null, $name = null)
+    {
+        $conn = $this->_connect();
+        if (is_a($conn, 'PEAR_Error')) {
+            return $conn;
+        }
+
+        $where = (is_null($path)) ? null : sprintf('WHERE vfs_path = %s', ((!strlen($path)) ? '""' : $this->_db->quote($this->_convertPath($path))));
+        $length_op = $this->_getFileSizeOp();
+        $sql = sprintf(
+            'SELECT SUM(%s(vfs_data)) FROM %s %s',
+            $length_op,
+            $this->_params['table'],
+            $where
+        );
+        $this->log($sql, PEAR_LOG_DEBUG);
+        $size = $this->_db->getOne($sql);
+
+        return $size !== null ? $size : 0;
+    }
+
     /**
      * Retrieve a file from the VFS.
      *
@@ -86,6 +149,52 @@ class VFS_sql extends VFS {
                                       'vfs_name' => $name));
     }
 
+    /**
+     * Retrieves a part of a file from the VFS. Particularly useful
+     * when reading large files which would exceed the PHP memory
+     * limits if they were stored in a string.
+     *
+     * @param string  $path       The pathname to the file.
+     * @param string  $name       The filename to retrieve.
+     * @param integer $offset     The offset of the part. (The new offset will be
+     *                            stored in here).
+     * @param integer $length     The length of the part. If the length = -1, the
+     *                            whole part after the offset is retrieved. If
+     *                            more bytes are given as exists after the given
+     *                            offset. Only the available bytes are read.
+     * @param integer $remaining  The bytes that are left, after the part that is
+     *                            retrieved.
+     *
+     * @return string The file data.
+     */
+    function readPart($path, $name, &$offset, $length = -1, &$remaining)
+    {
+        $conn = $this->_connect();
+        if (is_a($conn, 'PEAR_Error')) {
+            return $conn;
+        }
+
+        $data = $this->_readBlob($this->_params['table'], 'vfs_data',
+                                 array('vfs_path' => $this->_convertPath($path),
+                                       'vfs_name' => $name));
+
+        // Calculate how many bytes MUST be read, so the remainging
+        // bytes and the new offset can be calculated correctly.
+        $size = strlen ($data);
+        if ($length == -1 || (($length + $offset) > $size)) {
+            $length = $size - $offset;
+        }
+        if ($remaining < 0) {
+            $remaining = 0;
+        }
+
+        $data = substr($data, $offset, $length);
+        $offset = $offset + $length;
+        $remaining = $size - $offset;
+
+        return $data;
+    }
+
     /**
      * Stores a file in the VFS.
      *
@@ -99,16 +208,22 @@ class VFS_sql extends VFS {
      */
     function write($path, $name, $tmpFile, $autocreate = false)
     {
-        $size = filesize($tmpFile);
-        if ($size === 0) {
-            $data = '';
+        /* Don't need to check quota here since it will be checked when
+         * writeData() is called. */
+        if (function_exists('file_get_contents')) {
+            $data = file_get_contents($tmpFile);
         } else {
-            $dataFP = @fopen($tmpFile, 'rb');
-            if (!$dataFP) {
-                return PEAR::raiseError(sprintf(_("The temporary file \"%s\" does not exist."), $tmpFile));
+            $size = filesize($tmpFile);
+            if ($size === 0) {
+                $data = '';
+                } else {
+                $dataFP = @fopen($tmpFile, 'rb');
+                if (!$dataFP) {
+                    return PEAR::raiseError(sprintf(_("The temporary file \"%s\" does not exist."), $tmpFile));
+                }
+                $data = @fread($dataFP, $size);
+                @fclose($dataFP);
             }
-            $data = @fread($dataFP, $size);
-            @fclose($dataFP);
         }
 
         return $this->writeData($path, $name, $data, $autocreate);
@@ -126,6 +241,11 @@ class VFS_sql extends VFS {
      */
     function writeData($path, $name, $data, $autocreate = false)
     {
+        $res = $this->_checkQuotaWrite('string', $data);
+        if (is_a($res, 'PEAR_Error')) {
+            return $res;
+        }
+
         $conn = $this->_connect();
         if (is_a($conn, 'PEAR_Error')) {
             return $conn;
@@ -134,12 +254,12 @@ class VFS_sql extends VFS {
         $path = $this->_convertPath($path);
 
         /* Check to see if the data already exists. */
-        $sql = sprintf('SELECT vfs_id FROM %s WHERE vfs_path %s AND vfs_name = %s',
+        $sql = sprintf('SELECT vfs_id FROM %s WHERE vfs_path %s AND vfs_name = ?',
                        $this->_params['table'],
-                       (empty($path) && $this->_db->dbsyntax == 'oci8') ? ' IS NULL' : ' = ' . $this->_db->quote($path),
-                       $this->_db->quote($name));
+                       (empty($path) && $this->_db->dbsyntax == 'oci8') ? ' IS NULL' : ' = ' . $this->_db->quote($path));
+        $values = array($name);
         $this->log($sql, PEAR_LOG_DEBUG);
-        $id = $this->_db->getOne($sql);
+        $id = $this->_db->getOne($sql, $values);
 
         if (is_a($id, 'PEAR_Error')) {
             $this->log($id, PEAR_LOG_ERR);
@@ -191,6 +311,11 @@ class VFS_sql extends VFS {
      */
     function deleteFile($path, $name)
     {
+        $res = $this->_checkQuotaDelete($path, $name);
+        if (is_a($res, 'PEAR_Error')) {
+            return $res;
+        }
+
         $conn = $this->_connect();
         if (is_a($conn, 'PEAR_Error')) {
             return $conn;
@@ -198,13 +323,12 @@ class VFS_sql extends VFS {
 
         $path = $this->_convertPath($path);
 
-        $sql = sprintf('DELETE FROM %s WHERE vfs_type = %s AND vfs_path %s AND vfs_name = %s',
+        $sql = sprintf('DELETE FROM %s WHERE vfs_type = ? AND vfs_path %s AND vfs_name = ?',
                        $this->_params['table'],
-                       $this->_db->quote(VFS_FILE),
-                       (empty($path) && $this->_db->dbsyntax == 'oci8') ? ' IS NULL' : ' = ' . $this->_db->quote($path),
-                       $this->_db->quote($name));
+                       (empty($path) && $this->_db->dbsyntax == 'oci8') ? ' IS NULL' : ' = ' . $this->_db->quote($path));
+        $values = array(VFS_FILE, $name);
         $this->log($sql, PEAR_LOG_DEBUG);
-        $result = $this->_db->query($sql);
+        $result = $this->_db->query($sql, $values);
 
         if ($this->_db->affectedRows() == 0) {
             return PEAR::raiseError(_("Unable to delete VFS file."));
@@ -230,6 +354,18 @@ class VFS_sql extends VFS {
             return $conn;
         }
 
+        if (strpos($newpath, '/') === false) {
+            $parent = '';
+            $path = $newpath;
+        } else {
+            list($parent, $path) = explode('/', $newpath, 2);
+        }
+        if (!$this->isFolder($parent, $path)) {
+            if (is_a($result = $this->autocreatePath($newpath), 'PEAR_Error')) {
+                return $result;
+            }
+        }
+
         $oldpath = $this->_convertPath($oldpath);
         $newpath = $this->_convertPath($newpath);
 
@@ -340,12 +476,12 @@ class VFS_sql extends VFS {
         }
 
         /* All ok now delete the actual folder */
-        $sql = sprintf('DELETE FROM %s WHERE vfs_path %s AND vfs_name = %s',
+        $sql = sprintf('DELETE FROM %s WHERE vfs_path %s AND vfs_name = ?',
                        $this->_params['table'],
-                       (empty($path) && $this->_db->dbsyntax == 'oci8') ? ' IS NULL' : ' = ' . $this->_db->quote($path),
-                       $this->_db->quote($name));
+                       (empty($path) && $this->_db->dbsyntax == 'oci8') ? ' IS NULL' : ' = ' . $this->_db->quote($path));
+        $values = array($name);
         $this->log($sql, PEAR_LOG_DEBUG);
-        $delete = $this->_db->query($sql);
+        $delete = $this->_db->query($sql, $values);
         if (is_a($delete, 'PEAR_Error')) {
             $this->log($delete, PEAR_LOG_ERR);
             return PEAR::raiseError(sprintf(_("Unable to delete VFS directory: %s."), $delete->getMessage()));
@@ -375,9 +511,6 @@ class VFS_sql extends VFS {
 
         $path = $this->_convertPath($path);
 
-        $files = array();
-        $fileList = array();
-
         // Fix for Oracle not differentiating between '' and NULL.
         if (empty($path) && $this->_db->dbsyntax == 'oci8') {
             $where = 'vfs_path IS NULL';
@@ -385,15 +518,18 @@ class VFS_sql extends VFS {
             $where = 'vfs_path = ' . $this->_db->quote($path);
         }
 
-        $sql = sprintf('SELECT vfs_name, vfs_type, vfs_data, vfs_modified, vfs_owner FROM %s WHERE %s',
-                            $this->_params['table'],
-                            $where);
+        $length_op = $this->_getFileSizeOp();
+        $sql = sprintf('SELECT vfs_name, vfs_type, %s(vfs_data), vfs_modified, vfs_owner FROM %s WHERE %s',
+                       $length_op,
+                       $this->_params['table'],
+                       $where);
         $this->log($sql, PEAR_LOG_DEBUG);
         $fileList = $this->_db->getAll($sql);
         if (is_a($fileList, 'PEAR_Error')) {
             return $fileList;
         }
 
+        $files = array();
         foreach ($fileList as $line) {
             // Filter out dotfiles if they aren't wanted.
             if (!$dotfiles && substr($line[0], 0, 1) == '.') {
@@ -411,7 +547,7 @@ class VFS_sql extends VFS {
                     $file['type'] = VFS::strtolower($name[count($name) - 1]);
                 }
 
-                $file['size'] = VFS::strlen($line[2]);
+                $file['size'] = $line[2];
             } elseif ($line[1] == VFS_FOLDER) {
                 $file['type'] = '**dir';
                 $file['size'] = -1;
@@ -419,8 +555,8 @@ class VFS_sql extends VFS {
 
             $file['date'] = $line[3];
             $file['owner'] = $line[4];
-            $file['perms'] = '-';
-            $file['group'] = '-';
+            $file['perms'] = '';
+            $file['group'] = '';
 
             // filtering
             if ($this->_filterMatch($filter, $file['name'])) {
@@ -513,6 +649,32 @@ class VFS_sql extends VFS {
         return $folders;
     }
 
+    /**
+     * Garbage collect files in the VFS storage system.
+     *
+     * @param string $path   The VFS path to clean.
+     * @param integer $secs  The minimum amount of time (in seconds) required
+     *                       before a file is removed.
+     */
+    function gc($path, $secs = 345600)
+    {
+        $result = $this->_connect();
+        if (is_a($result, 'PEAR_Error')) {
+            return $result;
+        }
+
+        $sql = 'DELETE FROM ' . $this->_params['table']
+            . ' WHERE vfs_type = ? AND vfs_modified < ? AND (vfs_path = ? OR vfs_path LIKE ?)';
+        $this->log($sql, PEAR_LOG_DEBUG);
+
+        $values = array(VFS_FILE,
+                        time() - $secs,
+                        $this->_convertPath($path),
+                        $this->_convertPath($path) . '/%');
+
+        return $this->_db->query($sql, $values);
+    }
+
     /**
      * Renames all child paths.
      *
@@ -622,7 +784,16 @@ class VFS_sql extends VFS {
                 return $error;
             }
 
-            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+
+            // Set DB portability options.
+            switch ($this->_db->phptype) {
+            case 'mssql':
+                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
+                break;
+            default:
+                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            }
+
         }
 
         return true;
@@ -888,4 +1059,40 @@ class VFS_sql extends VFS {
         return $path;
     }
 
+    function _getFileSizeOp()
+    {
+        switch ($this->_db->dbsyntax) {
+        case 'mysql':
+            return 'LENGTH';
+
+        case 'oci8':
+            return 'LENGTHB';
+
+        case 'mssql':
+        case 'sybase':
+            return 'DATALENGTH';
+
+        case 'pgsql':
+        default:
+            return 'OCTET_LENGTH';
+        }
+    }
+
+    /**
+     * VFS_sql override of isFolder() to check for root folder.
+     *
+     * @param string $path  Path to possible folder
+     * @param string $name  Name of possible folder
+     *
+     * @return boolean        True if $path/$name is a folder
+     */
+    function isFolder($path, $name)
+    {
+        if ($path == '' && $name == '') {
+            // The root of VFS is always a folder.
+            return true;
+        }
+        return parent::isFolder($path, $name);
+    }
+
 }
