@@ -1,4 +1,4 @@
--- $Horde: horde/scripts/sql/create.mysql.sql,v 1.4.6.3 2005/07/11 18:12:58 chuck Exp $
+-- $Horde: horde/scripts/sql/create.mysql.sql,v 1.4.6.9 2006/07/05 15:53:10 jan Exp $
 --
 -- If you are installing Horde for the first time, you can simply
 -- direct this file to mysql as STDIN:
@@ -50,8 +50,10 @@ CREATE DATABASE horde;
 USE horde;
 
 CREATE TABLE horde_users (
-    user_uid       VARCHAR(255) NOT NULL,
-    user_pass      VARCHAR(255) NOT NULL,
+    user_uid                    VARCHAR(255) NOT NULL,
+    user_pass                   VARCHAR(255) NOT NULL,
+    user_soft_expiration_date   INT,
+    user_hard_expiration_date   INT,
 
     PRIMARY KEY (user_uid)
 );
@@ -67,6 +69,9 @@ CREATE TABLE horde_prefs (
     PRIMARY KEY (pref_uid, pref_scope, pref_name)
 );
 
+CREATE INDEX pref_uid_idx ON horde_prefs (pref_uid);
+CREATE INDEX pref_scope_idx ON horde_prefs (pref_scope);
+
 GRANT SELECT, INSERT, UPDATE, DELETE ON horde_prefs TO horde@localhost;
 
 CREATE TABLE horde_datatree (
@@ -78,7 +83,6 @@ CREATE TABLE horde_datatree (
        datatree_order INT,
        datatree_data TEXT,
        datatree_serialized SMALLINT DEFAULT 0 NOT NULL,
-       datatree_updated TIMESTAMP,
 
        PRIMARY KEY (datatree_id)
 );
@@ -121,7 +125,7 @@ CREATE TABLE horde_vfs (
     vfs_owner     VARCHAR(255) NOT NULL,
     vfs_data      LONGBLOB,
 
-    PRIMARY KEY   (vfs_id)
+    PRIMARY KEY (vfs_id)
 );
 
 CREATE INDEX vfs_path_idx ON horde_vfs (vfs_path);
@@ -129,6 +133,34 @@ CREATE INDEX vfs_name_idx ON horde_vfs (vfs_name);
 
 GRANT SELECT, INSERT, UPDATE, DELETE ON horde_vfs TO horde@localhost;
 
+CREATE TABLE horde_histories (
+    history_id       BIGINT NOT NULL,
+    object_uid       VARCHAR(255) NOT NULL,
+    history_action   VARCHAR(32) NOT NULL,
+    history_ts       BIGINT NOT NULL,
+    history_desc     TEXT,
+    history_who      VARCHAR(255),
+    history_extra    TEXT,
+
+    PRIMARY KEY (history_id)
+);
+
+CREATE INDEX history_action_idx ON horde_histories (history_action);
+CREATE INDEX history_ts_idx ON horde_histories (history_ts);
+CREATE INDEX history_uid_idx ON horde_histories (object_uid);
+
+GRANT SELECT, INSERT, UPDATE, DELETE ON horde_histories TO horde@localhost;
+
+CREATE TABLE horde_sessionhandler (
+    session_id             VARCHAR(32) NOT NULL,
+    session_lastmodified   INT NOT NULL,
+    session_data           LONGBLOB,
+
+    PRIMARY KEY (session_id)
+) ENGINE = InnoDB;
+
+GRANT SELECT, INSERT, UPDATE, DELETE ON horde_sessionhandler TO horde@localhost;
+
 FLUSH PRIVILEGES;
 
 -- Done!
