@@ -1,4 +1,4 @@
-// htmlArea v3.0 - Copyright (c) 2003-2004 dynarch.com
+// htmlArea v3.0 - Copyright (c) 2003-2005 dynarch.com
 //                               2002-2003 interactivetools.com, inc.
 // This copyright notice MUST stay intact for use (see license.txt).
 //
@@ -8,7 +8,7 @@
 // Version 3.0 developed by Mihai Bazon.
 //   http://dynarch.com/mishoo/
 //
-// $Id: htmlarea.js,v 1.95 2004/09/17 11:54:24 mishoo Exp $
+// $Id: htmlarea.js,v 1.97 2005/01/29 00:46:51 itools Exp $
 
 if (typeof _editor_url == "string") {
 	// Leave exactly one backslash at the end of _editor_url
@@ -37,6 +37,16 @@ HTMLArea.is_gecko  = (navigator.product == "Gecko");
 // Creates a new HTMLArea object.  Tries to replace the textarea with the given
 // ID with it.
 function HTMLArea(textarea, config) {
+        if (!HTMLArea._init_called) {
+	 alert("Error: You must called HTMLArea.init() first.  Like this:\n\n" +
+	       "HTMLArea.init();\n" +
+	       "HTMLArea.onload = function() {\n" +
+	       "  var editor = new HTMLArea('editor');\n" +
+	       "  editor.generate();\n" +
+	       "};\n");
+	  return;
+	}
+
 	if (HTMLArea.checkSupportedBrowser()) {
 		if (typeof config == "undefined") {
 			this.config = new HTMLArea.Config();
@@ -84,10 +94,12 @@ HTMLArea.init = function() {
 			head.appendChild(script);
 		} else {
 			document.title = savetitle;
+                        HTMLArea._init_called = 1;
 			HTMLArea.onload();
 		}
 	};
 	loadNextScript();
+
 };
 
 HTMLArea.loadScript(_editor_url + "dialog.js");
@@ -873,6 +885,7 @@ HTMLArea.prototype.generate = function () {
 			editor.onGenerate();
 	};
 	setTimeout(initIframe, 100);
+	return this;
 };
 
 // Switches editor mode; parameter can be "textmode" or "wysiwyg".  If no
@@ -1956,10 +1969,25 @@ HTMLArea.prototype._editorEvent = function(ev) {
 	}, 50);
 };
 
+HTMLArea.prototype.scrollToCaret = function() {
+	var
+		e = this.getParentElement(),
+		w = this._iframe.contentWindow,
+		h = w.innerHeight || w.height,
+		d = this._doc,
+		t = d.documentElement.scrollTop || d.body.scrollTop;
+	if (typeof h == "undefined")
+		return false;
+	if (e.offsetTop > h + t)
+		w.scrollTo(e.offsetLeft, e.offsetTop - h + e.offsetHeight);
+};
+
 HTMLArea.prototype.convertNode = function(el, newTagName) {
-	var newel = this._doc.createElement(newTagName);
+	var newel = this._doc.createElement(newTagName), p = el.parentNode;
 	while (el.firstChild)
 		newel.appendChild(el.firstChild);
+	p.insertBefore(newel, el);
+	p.removeChild(el);
 	return newel;
 };
 
@@ -2008,56 +2036,81 @@ HTMLArea.prototype.dom_checkBackspace = function() {
 };
 
 HTMLArea.prototype.dom_checkInsertP = function() {
-	var p = this.getAllAncestors();
-	var block = null;
-	var body = this._doc.body;
-	for (var i = 0; i < p.length; ++i)
+	var i, SC, left, right, r2,
+		sel   = this._getSelection(),
+		r     = this._createRange(sel),
+		p     = this.getAllAncestors(),
+		block = null,
+		doc   = this._doc,
+		body  = doc.body;
+
+	for (i = 0; i < p.length; ++i)
 		if (HTMLArea.isBlockElement(p[i]) && !/body|html/i.test(p[i].tagName)) {
 			block = p[i];
 			break;
 		}
-	var sel = this._getSelection();
-	var range = this._createRange(sel);
-	if (!range.collapsed)
-		range.deleteContents();
+
+	if (!r.collapsed)
+		r.deleteContents();
 	sel.removeAllRanges();
-	var SC = range.startContainer;
-	var SO = range.startOffset;
-	var EC = range.endContainer;
-	var EO = range.endOffset;
-	//alert(SC.tagName + ":" + SO + " => " + EC.tagName + ":" + EO);
-	if (SC == EC && SC == body && !SO && !EO) {
-		p = this._doc.createTextNode(" ");
-		body.insertBefore(p, body.firstChild);
-		range.selectNodeContents(p);
-		SC = range.startContainer;
-		SO = range.startOffset;
-		EC = range.endContainer;
-		EO = range.endOffset;
-	}
+	SC = r.startContainer;
+
 	if (!block) {
-		var r2 = range.cloneRange();
-		r2.setStartBefore(SC);
-		r2.setEndAfter(EC);
-		r2.surroundContents(block = this._doc.createElement("p"));
-		range.setEndAfter(block);
-		range.setStart(block.firstChild, SO);
-	} else range.setEndAfter(block);
-	var df = range.extractContents();
-	if (!/\S/.test(block.innerHTML))
-		block.innerHTML = "<br />";
-	p = df.firstChild;
-	if (!/\S/.test(p.innerHTML))
-		p.innerHTML = "<br />";
-	if (/^\s*<br\s*\/?>\s*$/.test(p.innerHTML) && /^h[1-6]$/i.test(p.tagName)) {
-		df.appendChild(this.convertNode(p, "p"));
-		df.removeChild(p);
+		left = SC;
+		for (i = SC; i && i != body && !HTMLArea.isBlockElement(i); i = HTMLArea.getPrevNode(i))
+			left = i;
+		right = SC;
+		for (i = SC; i && i != body && !HTMLArea.isBlockElement(i); i = HTMLArea.getNextNode(i))
+			right = i;
+		if (left != body && right != body) {
+			r2 = r.cloneRange();
+			r2.setStartBefore(left);
+			r2.surroundContents(block = doc.createElement("p"));
+			if (!/\S/.test(HTMLArea.getInnerText(block)))
+				block.innerHTML = "<br />";
+			block.normalize();
+			r.setEndAfter(right);
+			r.surroundContents(block = doc.createElement("p"));
+			if (!/\S/.test(HTMLArea.getInnerText(block)))
+				block.innerHTML = "<br />";
+			block.normalize();
+		} else {
+			r = doc.createRange();
+			r.setStart(body, 0);
+			r.setEnd(body, 0);
+			r.insertNode(block = doc.createElement("p"));
+			block.innerHTML = "<br />";
+		}
+		r.selectNodeContents(block);
+	} else {
+		r.setEndAfter(block);
+		var df = r.extractContents(), left_empty = false;
+		if (!/\S/.test(block.innerHTML)) {
+			block.innerHTML = "<br />";
+			left_empty = true;
+		}
+		p = df.firstChild;
+		if (p) {
+			if (!/\S/.test(HTMLArea.getInnerText(p))) {
+ 				if (/^h[1-6]$/i.test(p.tagName))
+ 					p = this.convertNode(p, "p");
+				p.innerHTML = "<br />";
+			}
+			if (/^li$/i.test(p.tagName) && left_empty && !block.nextSibling) {
+				left = block.parentNode;
+				left.removeChild(block);
+				r.setEndAfter(left);
+				r.collapse(false);
+				p = this.convertNode(p, /^[uo]l$/i.test(left.parentNode.tagName) ? "li" : "p");
+			}
+			r.insertNode(df);
+			r.selectNodeContents(p);
+		}
 	}
-	block.parentNode.insertBefore(df, block.nextSibling);
-	range.selectNodeContents(block.nextSibling);
-	range.collapse(true);
-	sel.addRange(range);
+	r.collapse(true);
+	sel.addRange(r);
 	this.forceRedraw();
+	this.scrollToCaret();
 };
 
 // retrieve the HTML
@@ -2402,6 +2455,20 @@ HTMLArea.getHTMLWrapper = function(root, outputRoot, editor) {
 	return html;
 };
 
+HTMLArea.getPrevNode = function(node) {
+	if (!node)                return null;
+	if (node.previousSibling) return node.previousSibling;
+	if (node.parentNode)      return node.parentNode;
+	return null;
+};
+
+HTMLArea.getNextNode = function(node) {
+	if (!node)            return null;
+	if (node.nextSibling) return node.nextSibling;
+	if (node.parentNode)  return node.parentNode;
+	return null;
+};
+
 HTMLArea.prototype.stripBaseURL = function(string) {
 	var baseurl = this.config.baseURL;
 
